Package: matrixcalc
Version: 1.0-6
Date: 2022-09-10
Title: Collection of Functions for Matrix Calculations
Author: Frederick Novomestky <fnovomes@poly.edu>
Maintainer: S. Thomas Kelly <tomkellygenetics@gmail.com>
Depends: R (>= 2.0.1)
Description: A collection of functions to support matrix calculations
        for probability, econometric and numerical analysis. There are
        additional functions that are comparable to APL functions which
        are useful for actuarial models such as pension mathematics.
        This package is used for teaching and research purposes at the
        Department of Finance and Risk Engineering, New York
        University, Polytechnic Institute, Brooklyn, NY 11201.
        Horn, R.A. (1990) Matrix Analysis. ISBN 978-0521386326.
        Lancaster, P. (1969) Theory of Matrices. ISBN 978-0124355507.
        Lay, D.C. (1995) Linear Algebra: And Its Applications. ISBN 978-0201845563.
License: GPL (>= 2)
BugReports: https://github.com/TomKellyGenetics/matrixcalc/issues
Packaged: 2022-09-14 20:41:38 UTC; tom
Repository: CRAN
Date/Publication: 2022-09-14 21:20:02 UTC
RoxygenNote: 7.1.2
NeedsCompilation: no
Built: R 4.6.0; ; 2026-04-26 09:44:46 UTC; unix

exportPattern( "^[[:alpha:]]+" )
export( "commutation.matrix",
        "creation.matrix",
        "D.matrix",
        "direct.prod",
        "direct.sum",
        "duplication.matrix",
        "E.matrices",
        "elimination.matrix",
        "entrywise.norm",
        "fibonacci.matrix",
        "frobenius.matrix",
        "frobenius.norm",
        "frobenius.prod",
        "H.matrices",
        "hadamard.prod",
        "hankel.matrix",
        "hilbert.matrix",
        "hilbert.schmidt.norm",
        "inf.norm",
        "is.diagonal.matrix",
        "is.idempotent.matrix",
        "is.indefinite",
        "is.negative.definite",
        "is.negative.semi.definite",
        "is.non.singular.matrix",
        "is.positive.definite",
        "is.positive.semi.definite",
        "is.singular.matrix",
        "is.skew.symmetric.matrix",
        "is.square.matrix",
        "is.symmetric.matrix",
        "K.matrix",
        "L.matrix",
        "lower.triangle",
        "lu.decomposition",
        "matrix.inverse",
        "matrix.power",
        "matrix.rank",
        "matrix.trace",
        "maximum.norm",
        "N.matrix",
        "one.norm",
        "pascal.matrix",
        "%s%",
        "set.submatrix",
        "shift.down",
        "shift.left",
        "shift.right",
        "shift.up",
        "spectral.norm",
        "stirling.matrix",
        "svd.inverse",
        "symmetric.pascal.matrix",
        "T.matrices",
        "toeplitz.matrix",
        "u.vectors",
        "upper.triangle",
        "vandermonde.matrix",
        "vec",
        "vech" )
        

hilbert.schmidt.norm <- function( x )
{
###
### This function is a wrapper function that uses the entrywise.norm function
### with p = 2 to obtain the fronbenius norm
###
### argument
### x = a numeric matrix or vector
###
    return( entrywise.norm( x, 2 ) )
}

hadamard.prod <- function( x, y )
{
###
### this function calculates the Hadamard product of two matrices x and y.
### the matrices must the same row and column order
###
### Parameters
### x = a numeric matrix object
### y = a numeric matrix object
###
    if ( !is.numeric( x ) ) {
        stop( "argument x is not numeric" )
    }    
    if ( !is.numeric( y ) ) {
        stop( "argument y is not numeric" )
    }    
    if ( is.matrix( x ) ) {
        Xmat <- x
    }
    else {
        if ( is.vector( x ) ) {
            Xmat <- matrix( x, nrow=length(x), ncol=1 )
        }
        else {
            stop( "argument x is neither a matrix or a vector" )
        }
    }    
    if ( is.matrix( y ) ) {
        Ymat <- y
    }
    else {
        if ( is.vector( y ) ) {
            Ymat <- matrix( y, nrow=length(x), ncol=1 )
        }
        else {
            stop( "argument x is neither a matrix or a vector" )
        }
    }    
    if ( nrow( Xmat ) != nrow( Ymat ) )
        stop( "argumentx x and y do not have the same row order" )
    if ( ncol( Xmat ) != ncol( Ymat ) )
        stop( "arguments x and y do not have the same column order" )
    return( Xmat * Ymat )
}

u.vectors <- function( n )
{
###
### This function constructs an identity matrix I of order
### p = n * ( n +1 ) / 2.  It also builds a lower triangular square matrix of
### order n.  The value of element [i,j] is the column number
### in the identify matrix.  It is the mapping from the coordinates
### to the column vector in the identity matrix.
###
### argument
### n = a positive integer value
###
    if ( n != trunc( n ) )
        stop( "argument n is not an integer" )
    if ( n < 2 )
        stop( "argument n is less than 2" )
    p <- n * ( n + 1 ) / 2
    I <- diag( rep( 1, p ) )
    k <- matrix( 0, nrow=n, ncol=n )
    for ( j in 1:n ) {
        for ( i in j:n ) {
            k[i,j] <- ( j - 1 ) * n + i - 0.5 * j * ( j - 1 )
        }
    }
    return( list( k=k, I=I ) )
}

K.matrix <- function( r, c=r )
{
###
### This function constructs the rc by rc commutation matrix
###
### Arguments
### r = a positive integer value for the number of rows in the H matrices
### c = a positive integer value for the number of columns in the H matrices
###
    if ( missing( r ) )
        stop( "argument r is missing" )
    if ( !is.numeric( r ) )
        stop( "argument r is not numeric" )
    if ( r != trunc( r ) )
        stop( "argument r is not an integer" )
    if ( r < 2 )
        stop( "argument r is less than 2" )
    if ( !is.numeric( c ) )
        stop( "argument c is not numeric" )
    if ( c != trunc( c ) )
        stop( "argument c is not an integer" )
    if ( c < 2 )
        stop( "argument c is less than 2" )
    H <- H.matrices( r, c )
    p <- r * c
    K <- matrix( 0, nrow=p, ncol=p )
    for ( i in 1:r ) {
        for ( j in 1:c ) {
            Hij <- H[[i]][[j]]
            K <- K + ( Hij %x% t( Hij ) )
        }
    }
    return( K )
}

is.singular.matrix <- function( x, tol=1e-8 )
{
###
### this function returns TRUE if the matrix argument x
### is singular and FALSE otherwise
###
### argument
### x = a numeric square matrix
### tol = tolerance level for zero
###
    if (!is.square.matrix( x ) )
        stop( "argument x is not a square matrix" )
    if ( !is.numeric( x ) )
        stop( "argument x is not a numeric matrix" )
    det.x <- det( x )
    return( abs( det.x ) < tol )
}

L.matrix <- function( n )
{
###
### This function constructs the elimination matrix as a mapping
### from vec(A) to vech(A)
###
### Arguments
### n = a positive integer value for the order of the matrix
###
    if ( missing( n ) )
        stop( "argument n is missing" )
    if ( !is.numeric( n ) )
        stop( "argument n is not numeric" )
    if ( n != trunc( n ) )
        stop( "argument n is not an integer" )
    if ( n < 2 )
        stop( "argument n is less than 2" )
    u <- u.vectors( n )
    E <- E.matrices( n )
    k <- u$k
    I <- u$I
    p <- n * ( n + 1 ) / 2
    nsq <- n * n
    L <- matrix( 0, nrow=p, ncol=nsq)
    for ( j in 1:n ) {
        for ( i in j:n ) {
            L <- L + I[,k[i,j]] %*% t( vec( E[[i]][[j]] ) )
        }
    }
    return( L )
}

matrix.power <- function( x, k )
{
###
### this function computes the k-th power of order n square matrix x
### if k is zero, the order n identity matrix is returned.  argument k
### must be an integer
###
### arguments
### x = a numeric square matrix
### k = an integer value exponent
###
    if ( !is.square.matrix( x ) )
        stop( "argument x is not a square matrix" )
    if ( !is.numeric( x ) )
        stop( "argument x is not numeric" )
    if ( !is.numeric( k ) )
        stop( "argument k is not numeric" )
    if ( !is.vector( k ) )
        stop( "argument k is not the proper data type" )
    if ( length( k ) > 1 )
        stop( "argument k is not a scalr number" )
    if ( k != trunc( k ) )
        stop( "argument k is not an integer" )
    n <- nrow( x )
    if ( k == 0 ) {
        return( diag( 1, n ) )
    }    
    if ( k < 0 ) {
        return( matrix.power( matrix.inverse(x), -k ) )
    }
    x.k <- x
    i <- 2
    while ( i <= k ) {
        x.k <- x.k %*% x
        i <- i + 1
    }
    return( x.k )
}

direct.sum <- function( x, y )
{
###
### This function computes the direct sum of two vectors or
### matrices resulting in a block diagonal matrix
###
### Arguments
### x = a numeric matrix or vector
### y = a numeric matrix or vector
###
    matrices <- list()
    if ( is.vector( x ) ) {
        A <- matrix( x )
    }
    else {
        if ( is.matrix( x ) ) {
            A <- x
        }
        else {
            stop( "Argument x is not a matrix or vector" )
        }
    }
    if ( is.vector( y ) ) {
        B <- matrix( y )
    }
    else {
        if ( is.matrix( y ) ) {
            B <- y
        }
        else {
            stop( "Argument y is not a matrix or vector" )
        }
    }
    C <- rbind( cbind( A, matrix( 0, nrow=nrow(A), ncol=ncol(B) ) ),
                cbind( matrix( 0, nrow=nrow(B), ncol=ncol(A) ), B ) )
    return( C )
}

lower.triangle <- function( x )
{
###
### this function returns the lower triangular matrix portion of matrix x
###
### Parameters
### x = a numeric matrix
###
    if ( !is.square.matrix( x ) )
        stop( "argument x is not a square numeric matrix" )
    y <- x
    y[row(x) < col(y)] <- 0
    return( y )
}

is.diagonal.matrix <- function( x, tol=1e-8 )
{
###
### this function returns TRUE if the off diagonal elements in absolute
### value are less than the given tolerance and FALSE otherwise
###
### arguments
### x = a numeric square matrix
### tol = tolerance level for zero
###
    if (!is.square.matrix( x ) )
        stop( "argument x is not a square matrix" )
    if ( !is.numeric( x ) )
        stop( "argument x is not a numeric matrix" )
    y <- x
    diag( y ) <- rep( 0, nrow( y ) )
    return( all( abs( y ) < tol ) )
}

fibonacci.matrix <- function( n )
{
###
### This function returns the order n + 1 Fibonacci matrix which
### is a square matrix derived from the Fibonacci sequence
###
### argument
### n = a positive integer
###
    if ( !is.numeric( n ) )
        stop( "argument n is not numeric" )
    if ( !is.vector( n ) )
        stop( "argument n is not the proper data type" )
    if ( length( n ) > 1 )
        stop( "argument n is not a scalr number" )
    if ( n != trunc( n ) )
        stop( "argument n is not an integer" )
    if ( n < 1 )
        stop( "argument n is less than 2" )
    np1 <- n + 1
    np2 <- n + 2
    f <- rep( 0, np2 )
    f[1] <- 1
    f[2] <- 1
    j <- 3
    while ( j <= np2 ) {
        f[j] <- f[j-1] + f[j-2]
        j <- j + 1
    }
    F <- matrix( 0, nrow=np1, ncol=np1 )
    for ( i in 0:n ) {
        ip1 <- i + 1
        for ( j in 0:n ) {
            jp1 <- j + 1
            if ( i - j + 1 >= 0 ) {
                F[ip1,jp1] <- f[i - j + 2]
            }
        }
    }
    return( F )
}

is.skew.symmetric.matrix <- function ( x, tol=1e-8 )
{
###
### this function returns TRUE if the matrix argument x is
### skew symmetric and FALSE otherwise
###
### argument
### x = a numeric square matrix
### tol = tolerance level for zero
###
    if ( !is.square.matrix( x ) )
        stop( "argument x is not a square matrix" )
    if ( !is.numeric( x ) )
        stop( "argument x is not a numeric matrix" )
    return( all( abs( x + t(x) ) < tol ) )
}

creation.matrix <- function( n )
{
###
### this function returns an order n creation matrix that is useful
### in numerical mathematics such as the solution of differential equations.
###
### argument
### n = a positive integer greater than 1
###
    if ( !is.numeric( n ) )
        stop( "argument n is not numeric" )
    if ( !is.vector( n ) )
        stop( "argument n is not the proper data type" )
    if ( length( n ) > 1 )
        stop( "argument n is not a scalr number" )
    if ( n != trunc( n ) )
        stop( "argument n is not an integer" )
    if ( n < 2 )
        stop( "argument n is less than 2" )
    H <- shift.down( diag( seq( 1, n ) ) )
    return( H )
}

T.matrices <- function( n )
{
###
### This function constructs a list of lists.  For each high level list i,
### the component is a list of n components.  These components are derived
### from the list of lists E generated by the function E.matrices
###
### Argument
### n = a positive integer for the order of an underlying square matrix
###
    if( missing( n ) )
        stop( "argument n is missing" )
    if ( !is.numeric( n ) )
        stop( "argument n is not numeric" )
    if ( n != trunc( n ) )
        stop( "argument n is not an integer" )
    if ( n < 2 )
        stop( "argument n is less than 2" )
    E <- E.matrices( n )
    T <- list()
    for ( i in 1:n ) {
        T[[i]] <- list()
        for ( j in 1:n ) {
            if ( i == j ) {
                T[[i]][[j]] <- E[[i]][[j]]
            }    
            else {
                T[[i]][[j]] <- E[[i]][[j]] + E[[j]][[i]]
            }    
        }
    }
    return( T )
}

entrywise.norm <- function( x, p )
{
###
### This function computes the entrywise norm in which the absolute value of
### each element of matrix x is raised to the power p and the norm is the
### sum of these quantities
###
### arguments
### x = a numeric matrix or vector
### p = a real value for the power
###
    if ( !is.numeric( x ) ) {
        stop( "argument x is not numeric" )
    }
    if ( is.matrix( x ) ) {
        Xmat <- x
    }
    else {
        if ( is.vector( x ) ) {
            Xmat <- matrix( x, nrow=length(x), ncol=1 )
        }
        else {
            stop( "argument x is neither vector nor matrix" )
        }
    }
    if ( p == 0 ) {
        stop( "exponent p is zero" )
    }
    if ( is.infinite( p ) ) {
        return( maximum.norm( x ) )
    }    
    return( ( sum( abs( Xmat ) ^ p ) ) ^ ( 1 / p ) )
}

duplication.matrix <- function( n=1 )
{
###
### this function returns a matrix sith n * n rows and n * (n + 1 ) / 2
### columns that transforms vech( A ) to vec( A ) where A is a symmetric n by n matrix
###
### Parameter
### n = the order of the matrix
###
    return( D.matrix( n ) )
}

stirling.matrix <- function( n )
{
###
### this function constructs the Stirling matrix, a lower triangular matrix containing
### the Stirling numbers of the second kind
###
### argument
### n = a positive integer for the order of the matrix
###
    if ( !is.numeric( n ) )
        stop( "argument n is not numeric" )
    if ( !is.vector( n ) )
        stop( "argument n is not of the proper data type" )
    if ( length( n ) != 1 )
        stop( "argument n is not a scalar integer" )
    if ( n < 1 )
        stop( "arguemtn n is less than 1" )
    if ( n == 1 ) {
        return( S )
    }
    nm1 <- n - 1
    np1 <- n + 1
    S <- matrix( 0, nrow=np1, ncol=np1 )
    for ( i in 0:n ) {
        ii <- i + 1
        S[ii,ii] <- 1
    }
    for ( j in 0:n ) {
        jj <- j + 1
        S[1,jj] <- 0
        S[jj,1] <- 0
    }    
    for ( i in 1:nm1 ) {
        ii <- i + 1
        for ( j in 1:i ) {
            jj <- j + 1
            S[ii+1,jj] <- S[ii,jj-1] + j * S[ii,jj]
        }
    }
    S[1,1] <- 1
    return( S )
}

toeplitz.matrix <- function( n, x )
{
###
### this function returns the order n Toeplitz matrix derived from
### the order 2 * n - 1 vector
###
### arguments
### n = a positive integer value for the order of the Hankel matrix
### x = an order 2 * n - 1 vector of numeric values
###
    if ( !is.numeric( n ) )
        stop( "argument n is not numeric" )
    if ( n != trunc( n ) )
        stop( "argument n ix not an integer" )
    if ( n < 2 )
        stop( "argument n is less than 2" )
    if ( !is.vector( x ) )
        stop( "argument x is not a vector" )
    m <- length( x )
    if ( m != ( 2 * n - 1 ))
        stop( "length of argument x is not comparable with n n" )
    T <- matrix( 0, nrow=n, ncol=n )
    for ( i in 1:n ) {
        for ( j in 1:n ) {
            T[i,j] <- x[(i - j) + n]
        }
    }
    return( T )
}

H.matrices <- function( r, c=r )
{
###
### This function returns a list of lists.  The length of the high
### level list is r.  Each component i of the high level list is
### a list of c components.  Each sub-component j of main component i
### is an r by c matrix.  Each matrix is the outer product of column
### i in the r by r identity and column j of the c by c identity matrix
###
### Arguments
### r = a positive integer for the number of rows in each matrix
### c = a positive integer for the number of columns in each matrix
###
    if ( missing( r ) )
        stop( "argument r is missing" )
    if ( !is.numeric( r ) )
        stop( "argument r is not numeric" )
    if ( r != trunc( r ) )
        stop( "argument r is not an integer" )
    if ( r < 2 )
        stop( "argument r is less than 2" )
    if ( !is.numeric( c ) )
        stop( "argument c is not numeric" )
    if ( c != trunc( c ) )
        stop( "argument c is not an integer" )
    if ( c < 2 )
        stop( "argument c is less than 2" )
    Ir <- diag( rep( 1, r ) )
    Ic <- diag( rep( 1, c ) )
    H <- list()
    for ( i in 1:r) {
        H[[i]] <- list()
        for ( j in 1:c ) {
            H[[i]][[j]] <- Ir[i,] %o% Ic[j,]
        }
    }
    return( H )
}

hilbert.matrix <- function(n) 
{   
###
### this function returns an n by n Hilbert matrix
###
### Parameter
### n = the row (column) dimension of the matrix
###
    if ( n != trunc( n ) )
        stop( "argument n is not an integer" )
    if ( n < 2 )
        stop( "argument n is less than 2" )
    i <- 1:n
    X <- 1 / outer(i - 1, i, "+")
    return( X  )
}

spectral.norm <- function( x )
{
###
### This function computes the spectral norm for an m by n matrix x
### If x is a vector, then the L2 norm is returned
###
### arguments
### x = a numeric matrix or vector
###
    if ( !is.numeric( x ) ) {
        stop( "argument x is not numeric" )
    }
    if ( is.vector( x ) ) {
        return( sqrt( sum( x * x ) ) )
    }
    if ( !is.matrix( x ) ) {
        return( "argument x is not a matrix" )
    }
    A <- t(x) %*% x
    eigenA <- eigen( A )
    lambdaA <- eigenA$values
    maxLambdaA <- lambdaA[1]
    if ( maxLambdaA < 0 ) {
        stop( "t(x) %*% x is negative definite" )
    }
    return( sqrt( maxLambdaA ) )
}

commutation.matrix <- function( r, c=r )
{
###
### this function returns a square matrix with p = r * c rows and columns
###
### Parameters
### m = integer rows
### n = integer columns
###
    return( K.matrix( r, c ) )
}

is.square.matrix <- function( x )
{
###
### determines if the given matrix is a square matrix
###
### arguments
### x = a matrix object
###
    if ( !is.matrix( x ) )
        stop( "argument x is not a matrix" )
    return( nrow(x) == ncol(x) )
}

elimination.matrix <- function( n )
{
###
### this function returns a matrix sith n * n comuns and n * (n + 1 ) / 2
### rows that transforms vec( A ) to vech( A ) where A is a symmetric n by n matrix
###
### Parameter
### n = the order of the matrix
###
    return( L.matrix( n ) )
}

pascal.matrix <- function( n )
{
###
### this function returns an n by n Pascal matrix
###
### Parameter
### n = the row( column ) dimension of the matrix
###
    S <- symmetric.pascal.matrix( n )
    luS <- lu.decomposition( S )
    P <- luS$L
    return( P )
}

matrix.rank <- function ( x, method=c("qr", "chol" ) )
{
###
### this function returns the rank of a square matrix based on the selected method
###
### Parameter
### x = a square numeric matrix
### method = a character string that defines the method
###
    if ( !is.square.matrix( x ) )
        stop( "argument x is not a square matrix" )
    method = method[1]
    if (method == "chol") {
        ans = attr(chol(x, pivot = TRUE), "rank") 
    } else {
        ans = qr(x)$rank 
    }
    
    # Return Value:
    return( ans  )
}

maximum.norm <- function( x )
{
###
### This function computes and returns the maximum norm of matrix x
###
### arguments
### x = a numeric matrix or vector
###
    if ( !is.numeric( x ) ) {
        stop( "argument x is not numeric" )
    }
    if ( is.matrix( x ) ) {
        Xmat <- x
    }
    else {
        if ( is.vector( x ) ) {
            X.mat <- x
        }
        else {
            stop( "argument is neither a matrix nor a vector" )
        }
    }
    norm <- max( abs( Xmat ) )
    return( norm )
}

matrix.trace <- function( x )
{
###
### this function returns the trace of the given square matrix
###
### parameters
### x = a numeric square matrix object
###
    if ( !is.square.matrix( x ) )
        stop( "argument x is not a square matrix" )
    return( sum( diag( x ) ) )
}

vech <- function( x )
{
###
### returns a stack of the lower triangular matrix as a matrix with 1 column
###
### Parameters
### x = a numeric matrix square matrix
###
    if ( !is.square.matrix( x ) )
        stop( "argument x is not a square numeric matrix" )
    return( t( t( x[!upper.tri(x)] ) ) )
}

frobenius.prod <- function( x, y )
{
###
### this function calculates the Frobenius inner product product of two matrices x and y.
### the matrices must the same row and column order
###
### Parameters
### x = a numeric matrix or vector object
### y = a numeric matrix or vector object
###
    return( sum( hadamard.prod(x, y) ) )
}

is.symmetric.matrix <- function( x )
{
###
### this function determines if the matrix is symmetric
###
### argument
### x = a numeric matrix object
###
    if ( !is.matrix( x ) ) {
        stop( "argument x is not a matrix" )
    }
    if ( !is.numeric( x ) ) {
        stop( "argument x is not a numeric matrix" )
    }    
    if ( !is.square.matrix( x ) )
        stop( "argument x is not a square numeric matrix" )
    return( sum( x == t(x) ) == ( nrow(x) ^ 2 ) )
}

shift.right <- function( A, cols = 1, fill = 0 )
{
###
### this function returns a matrix that has been shifted to the right m columns
### filling the previous columns with the given fill value
###
### Arguments
### A = a numerical matrix
### cols = number of cols to be shifed to the right
### fill = a numeric value to be used to fill the cols
###
    if ( !is.matrix( A ) ) {
        stop( "argument A is not a matrix" )
    }
    if ( !is.numeric( A ) ) {
        stop( "argument A is not a numeric matrix" )
    }    
    if ( cols < 0 )
        stop( "Argument cols is not positive" )
    if ( cols != trunc( cols ) )
        stop( "Arguments cols is not an integer" )
    if ( !is.numeric( fill ) )
        stop( "Argument fill is not numeric" )
    if ( cols > 0 )
        return( shift.right( cbind( rep( fill, nrow(A) ), A[,1:ncol(A)-1] ),
                cols - 1, fill ) )
    return( A )    
}

"%s%" <- function( x, y )
{
###
### This function is a wrapper the direct sum of two vectors or
### matrices resulting in a block diagonal matrix
###
### Arguments
### x = a numeric matrix or vector
### y = a numeric matrix or vector
###
    return( direct.sum( x, y ) )
}

lu.decomposition <- function( x )
{
###
### This function performs an LU decomposition of the given square matrix argument
### the results are returned in a list of named components.  The Doolittle decomposition
### method is used to obtain the lower and upper triangular matrices
###
### arguments
### x = a square numeric matrix
###
    if ( !is.square.matrix( x ) )
        stop( "argument x is not a square matrix" )
    if ( !is.numeric( x ) )
        stop( "argument x is not numeric" )
    n <- nrow( x )
    L <- matrix( 0, nrow=n, ncol=n )
    U <- matrix( 0, nrow=n, ncol=n )
    diag( L ) <- rep( 1, n )
    for ( i in 1:n ) {
        ip1 <- i + 1
        im1 <- i - 1
        for ( j in 1:n ) {
            U[i,j] <- x[i,j]
            if ( im1 > 0 ) {
                for ( k in 1:im1 ) {
                    U[i,j] <- U[i,j] - L[i,k] * U[k,j]
                }
            }
        }
        if ( ip1 <= n ) {
            for ( j in ip1:n ) {
                L[j,i] <- x[j,i]
                if ( im1 > 0 ) {
                    for ( k in 1:im1 ) {
                        L[j,i] <- L[j,i] - L[j,k] * U[k,i]
                    }
                }
                if ( U[i,i] == 0 )
                    stop( "argument x is a singular matrix" )
                L[j,i] <- L[j,i] / U[i,i]
            }    
        }
    }
    result <- list( L=L, U=U )
    return( result )
}

vec <- function( x )
{
###
### this function returns a column vector that is a stack of the columns of x
###
### Parameters
### x = a numeric matrix
###
    if ( !is.matrix( x ) ) {
        stop( "argument x is not a matrix" )
    }
    if ( !is.numeric( x ) ) {
        stop( "argument x is not a numeric matrix" )
    }    
    return( t( t( as.vector( x ) ) ) )
}

svd.inverse <- function( x )
{
###
### This function returns the Penrose-Moore pseudo inverse matrix
### for the given n by p matrix A using singular value decomposition
###
### arguments
### x = a numeric matrix or vector
### nu = positive integer for the number of left singular vectors to be computed
### nv = positive integer for the number of right singular vectors to be computed
### LINKPAC = logical value to determine if LINKPACK should be used.
###
    if ( !is.numeric( x ) ) {
        stop( "argument x is not numeric" )
    }
    if ( is.matrix( x ) ) {
        Xmat <- x
    }
    else {
        if ( is.vector( x ) ) {
            Xmat <- matrix( x, nrow=length( x ), ncol=1 )
        }
        else {
            stop( "argument x is neither a vector nor a matrix" )
        }
    }
    svdXmat <- svd( Xmat )
    U <- svdXmat$u
    d <- svdXmat$d
    if ( any( d == 0 ) ) {
        stop( "x has at least one zero singular value" )
    }
    if ( length( d ) == 1 ) {
        Dinv <- matrix( 1 / d, nrow=1 )
    }
    else {
        Dinv <- diag( 1 / d )
    }    
    V <- svdXmat$v
    Xinv <- V %*% Dinv %*% t( U )
    return( Xinv )
}

is.non.singular.matrix <- function( x, tol=1e-8 )
{
###
### This function determines if a square matrix is non singular by
### evaluating the determinant
###
### arguments
### x = a numeric square matrix
### tol = tolerance level for zero
###
    if (!is.square.matrix( x ) )
        stop( "argument x is not a square matrix" )
    if ( !is.numeric( x ) )
        stop( "argument x is not a numeric matrix" )
    det.x <- det( x )
    return( abs( det.x ) >= tol )
}

inf.norm <- function( x )
{
###
### This function returns the p = Inf norm wich is the maximum absolute row sum
### of the matrix x
###
### argument
### x = a numeric vector or matrix
###
    if ( !is.numeric( x ) ) {
        stop( "argument x is not numeric" )
    }
    if ( is.matrix( x ) ) {
        Xmat <- x
    }
    else {
        if ( is.vector( x ) ) {
            Xmat <- matrix( x, nrow=length(x), ncol=1 )
        }
        else {
            stop( "argument x is neither a matrix nor a vector" )
        }
    }
    norm <- max( apply( abs( Xmat ), 1, sum ) )
    return( norm )
}

vandermonde.matrix <- function( alpha, n )
{
###
### this function returns an m by n matrix of the powers of the alpha vector
###
### Parameters
### alpha = an m dimensional vector
### n = an integer
###
    if ( !is.vector( alpha ) )
        stop( "argument alpha is not a vector" )
    if ( !is.numeric( alpha ) )
        stop( "argument n is not a numeric vector" )
    m <- length( alpha )
    V <- matrix( 0, nrow=m, ncol=n )
    V[,1] <- rep( 1, m )
    j <- 2
    while ( j <= n ) {
       x <- alpha ^ ( j - 1 )
       V[,j] <- x
       j <- j + 1
   }
   return( V )
}

N.matrix <- function( n )
{
###
### This function returns the n*n by n*n matrix that is the sum of the
### implicit commutation matrix Kn and an identity matrix
###
### Arguments
### n = a positive integer value for the underlying matrix
###
    if ( missing( n ) )
        stop( "argument n is missing" )
    if ( !is.numeric( n ) )
        stop( "argument n is not numeric" )
    if ( n != trunc( n ) )
        stop( "argument n is not an integer" )
    if ( n < 2 )
        stop( "argument n is less than 2" )
    return( ( K.matrix( n ) + diag( rep( 1, n * n ) ) ) / 2 )
}

matrix.inverse <- function( x )
{
###
### this function returns the inverse of a square matrix
###
### Parameters
### x = a square numeric matrix
###
    if ( !is.square.matrix( x ) )
        stop( "argument x is not a square matrix" )
    return( solve( x ) )
}

shift.up <- function( A, rows = 1, fill = 0 )
{
###
### this function returns a matrix that has been shifted up m rows
### filling the previous rows with the given fill value
###
### Arguments
### A = a numerical matrix
### rows = number of rows to be shifed upwards
### fill = a numeric value to be used to fill the rows
###
    if ( !is.matrix( A ) ) {
        stop( "argument A is not a matrix" )
    }
    if ( !is.numeric( A ) ) {
        stop( "argument A is not a numeric matrix" )
    }    
    if ( rows != trunc( rows ) )
        stop( "Arguments rows is not an integer" )
    if ( rows < 0 )
        stop( "Argument rows is not positive" )
    if ( !is.numeric( fill ) )
        stop( "Argument fill is not numeric" )
    if ( rows > 0 )
        return( shift.up( rbind( A[2:nrow(A),], rep( fill, ncol(A) ) ),
                rows - 1, fill ) )
    return( A )    
}

is.idempotent.matrix <- function( x, tol=1e-8 )
{
###
### This function returns TRUE if the argument matrix is idempotent,
### that is, the product of the matrix with itself returns the same
### matrix within the prescribed tolerance and FALSE otherwise
###
### arguments
### x = a numeric square matrix
### tol = tolerance level for zero
###
    if (!is.square.matrix( x ) )
        stop( "argument x is not a square matrix" )
    if ( !is.numeric( x ) )
        stop( "argument x is not a numeric matrix" )
    return( all( abs( x %*% x - x ) < tol ) )
}

set.submatrix <- function( x, y, row, col )
{
###
### Returns a matrix where y has been inserted into x at the given row and column
###
### Arguments
### x = a matrix object
### y = a matrix object
### row = an integer row number
### col = an integer column number
###
    if ( !is.matrix( x ) )
        stop( "argument x is not a matrix" )
    if ( !is.numeric( x ) )
        stop( "argument x is not a numeric matrix" )
    if ( !is.matrix( y ) )
        stop( "argument y is not a matrix" )
    if ( !is.numeric( y ) )
        stop( "argument y is not a numeric matrix" )
    if ( row <= 0 )
        stop( "argument row is not positive" )
    if ( row != trunc( row ) )
        stop( "argument row is not an integer" )
    if ( col <= 0 )
        stop( "argument col is not positive" )
    if ( col != trunc( col ) )
        stop( "argument col is not an integer" )
    row.range <- row:(row+nrow(y)-1)
    col.range <- col:(col+ncol(y)-1)
    x.row.range <- 1:nrow(x)
    x.col.range <- 1:ncol(x)
    if ( sum( row.range %in% x.row.range ) != length(row.range) )
        stop( "row range not inside row of argument x" )
    if ( sum( col.range %in% x.col.range ) != length(col.range) )
        stop( "col range not inside the column of argument x" )
    z <- x
    z[row.range,col.range] <- y
    return( z )
}

D.matrix <- function( n )
{
###
### This function constructs the linear transformation D that maps vech(A)
### to vec(A) when A is a symmetric matrix
###
### Arguments
### n = a positive integer for the order of a matrix
###
    if( missing( n ) )
        stop( "argument n is missing" )
    if ( !is.numeric( n ) )
        stop( "argument n is not numeric" )
    if ( n != trunc( n ) )
        stop( "argument n is not an integer" )
    if ( n < 2 )
        stop( "argument n is less than 2" )
    p <- n * ( n + 1 ) /2
    nsq <- n * n
    Dt <- matrix( 0, nrow=p, ncol=nsq )
    T <- T.matrices( n )
    u <- u.vectors( n )
    k <- u$k
    I <- u$I
    for ( j in 1:n ) {
        for ( i in j:n ) {
            Dt <- Dt + I[,k[i,j]] %*% t( vec( T[[i]][[j]] ) )
        }
    }
    return( t( Dt ) )
}

E.matrices <- function( n )
{
###
### This function creates a list of lists.  The number of components
### in the higher level list is n.  For each component i in the
### higher level list, the number of components in the sub-list
### is n.  Each component j of the sub-list is an n by n matrix
### e_i %*% t ( e_j ).  Each of the arrays is a vector in an
### n by n identity matrix
###
### argument
### n = a positive integer value greater than or equal to 2
###
    if( missing( n ) )
        stop( "argument n is missing" )
    if ( !is.numeric( n ) )
        stop( "argument n is not numeric" )
    if ( n != trunc( n ) )
        stop( "argument n is not an integer" )
    if ( n < 2 )
        stop( "argument n is less than 2" )
    I <- diag( rep( 1, n ) )
    E <- list()
    for ( i in 1:n ) {
        E[[i]] <- list()
        for ( j in 1:n ) {
            E[[i]][[j]] <- I[i,] %o% I[j,]
        }
    }
    return( E )
}

hankel.matrix <- function( n, x )
{
###
### This function returns an order n Hankel matrix from the values in
### the order m vector x derived by a circular shift of the values
### in this vector to the left.  Hankel matrices are formed when a given sequence
### of output data and a realization of an underlying state-space or
### hidden Markov model is desired.
###
### arguments
### n = a positive integer value for the order of the Hankel matrix
### x = an order 2 * n + 1 vector of numeric values
###
    if ( !is.numeric( n ) )
        stop( "argument n is not numeric" )
    if ( n != trunc( n ) )
        stop( "argument n ix not an integer" )
    if ( n < 2 )
        stop( "argument n is less than 2" )
    if ( !is.vector( x ) )
        stop( "argument x is not a vector" )
    m <- length( x )
    if ( m < n )
        stop( "length of argument x is less than n" )
    y <- x
    H <- matrix( 0, nrow=n, ncol=n )
    H[1,] <- y[1:n]
    for ( i in 2:n ) {
        y <- c( y[2:m], y[1] )
        H[i,] <- y[1:n]
    }
    return( H )
}

    

symmetric.pascal.matrix <- function( n )
{
###
### this function returns an n by n Pascal matrix
###
### Parameter
### n = the row( column ) dimension of the matrix
###
    if ( n <= 0 )
        stop( "argument n is not positive" )
    if ( n != trunc( n ) )
        stop( "argument n is not an integer" )
    nm1 = n-1
    n.over.r <- function(n, r) { prod(1:n) / (prod(1:(n-r)) * prod(1:r) ) }
    X <- rep(1, nm1)
    for ( i in 1:nm1 )
        for ( j in 1:nm1 )
            X <- c(X, n.over.r(i+j, j))
    X <- cbind(rep(1, nm1+1), matrix(X, byrow = TRUE, ncol = nm1))
    return( X  )
}

direct.prod <- function( x, y )
{
###
### This function computes the direct product of two vectors or
### matrices resulting in a matrix
###
### Arguments
### x = a numeric matrix or vector
### y = a numeric matrix or vector
###
    if ( is.vector( x ) ) {
        A <- matrix( x )
    }
    else {
        if ( is.matrix( x ) ) {
            A <- x
        }
        else {
            stop( "Argument x is not a matrix or vector" )
        }
    }
    if ( is.vector( y ) ) {
        B <- matrix( y )
    }
    else {
        if ( is.matrix( y ) ) {
            B <- y
        }
        else {
            stop( "Argument y is not a matrix or vector" )
        }
    }
###
### construct a list of lists.  The number of elements in the higher level
### is the number of rows in A.  The number of elements in the lower level
### is the number of columns in A.  The value for higher level index i
### and lower level index j is A[i,j] * B
###
    matrices <- list()
    for ( i in 1:nrow( A ) ) {
        matrices[[i]] <- list()
        for ( j in 1:ncol( A ) ) {
            matrices[[i]][[j]] <- A[i,j] * B
        }
    }
###
### construct a list of matrices.  The number of elements is the number
### of rows in A.  The value for element i is the column binding of all the elements
### in the higher level matrices[[i]].
###
    row.matrices <- list()
    for ( i in 1:nrow( A ) ) {
        row.matrices[[i]] <- matrices[[i]][[1]]
        if ( ncol( A ) > 1 ) {
            for ( j in 2:ncol( A ) ) {
                row.matrices[[i]] <- cbind( row.matrices[[i]], matrices[[i]][[j]] )
            }
        }
    }
###
### construct the result matrix C as the row binding of all of the elements in row.matrices
###
    C <- row.matrices[[1]]
    if ( length( row.matrices ) > 1 ) {
        for ( i in 2:length( row.matrices ) ) {
            C <- rbind( C, row.matrices[[i]] )
        }
    }    
    return( C )
}

is.indefinite <- function( x, tol=1e-8 )
{
###
### this function determines if the given real symmetric matrix is indefinite,
### that is, the matrix is neither positive definite nor negative definite.
###
### parameters
### x = a square numeric matrix object
### tol = tolerance level for zero
###
    if ( !is.square.matrix( x ) )
        stop( "argument x is not a square matrix" )
    if ( !is.symmetric.matrix( x ) )
        stop( "argument x is not a symmetric matrix" )
    if ( !is.numeric( x ) )
        stop( "argument x is not a numeric matrix" )
    eigenvalues <- eigen(x, only.values = TRUE)$values
    n <- nrow( x )
    for ( i in 1: n ) {
        if ( abs( eigenvalues[i] ) < tol ) {
            eigenvalues[i] <- 0
        }
    }
    if( any( eigenvalues > 0 ) && any( eigenvalues < 0 ) ) {
        return( TRUE )
    }
    return( FALSE )
}

\name{vech}
\alias{vech}
\title{ Vectorize a matrix }
\description{
  This function returns a stack of the lower triangular matrix of a square matrix as a matrix with 1 column
  and n * ( n + 1 ) / 2 rows
}
\usage{
vech(x)
}
\arguments{
  \item{x}{ a matrix }
}
\value{
  A matrix with \eqn{\frac{1}{2}n\left( {n + 1} \right)} rows and one column.
}
\references{
  Magnus, J. R. and H. Neudecker (1999) \emph{Matrix Differential Calculus with Applications in Statistics and Econometrics},
  Second Edition, John Wiley.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\seealso{
  \code{\link{is.square.matrix}}
}
\examples{
x <- matrix( seq( 1, 16, 1 ), nrow=4, byrow=TRUE )
print( x )
y <- vech( x )
print( y )
}
\keyword{ math }

\name{creation.matrix}
\alias{creation.matrix}
\title{ Creation Matrix }
\description{
  This function returns the order n creation matrix, a square matrix with the
  sequence 1, 2, ..., n - 1 on the sub-diagonal below the principal diagonal.
}
\usage{
creation.matrix(n)
}
\arguments{
  \item{n}{ a positive integer greater than 1 }
}
\details{
  The order \eqn{n} creation matrix is also called the derivation matrix and is
  used in numerical mathematics and physics. It arises in the solution of linear
  dynamical systems.  The form of the matrix is 
  \eqn{\left\lbrack {\begin{array}{cccccc}
0&0&0& \cdots &0&0\\
1&0&0& \cdots &0&0\\
0&2&0& \cdots &0&0\\
0&0&3& \ddots &0&0\\
 \vdots & \vdots & \vdots & \ddots & \ddots &{}\\
0&0&0& \cdots &{n - 1}&0
\end{array}} \right\rbrack}.
}
\value{
  An order \eqn{n} matrix.
}
\references{
  Aceto, L. and D. Trigiante (2001). Matrices of Pascal and Other Greats,
  \emph{American Mathematical Monthly}, March 2001, 108(3), 232-245.
  
  Weinberg, S. (1995). \emph{The Quantum Theory of Fields}, Cambridge
  University Press.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  If the argument n is not an integer that is greater than 1, 
  the function presents an error message and stops.
}
\examples{
H <- creation.matrix( 10 )
print( H )
}
\keyword{ math }

\name{is.idempotent.matrix}
\alias{is.idempotent.matrix}
\title{ Test for idempotent square matrix }
\description{
  This function returns a \code{TRUE} value if the square matrix argument x
  is idempotent, that is, the product of the matrix with itself is the matrix.
  The equality test is performed to within the specified tolerance level.  If
  the matrix is not idempotent, then a \code{FALSE} value is returned.
}
\usage{
is.idempotent.matrix(x, tol = 1e-08)
}
\arguments{
  \item{x}{ a numeric square matrix }
  \item{tol}{ a numeric tolerance level usually left out }
}
\details{
  Idempotent matrices are used in econometric analysis.  Consider the problem of
  estimating the regression parameters of a standard linear model 
  \eqn{{\bf{y}} = {\bf{X}}\;{\bf{\beta }} + {\bf{e}}} using the method of least squares.
  \eqn{{\bf{y}}} is an order \eqn{m} random vector of dependent variables. 
  \eqn{{\bf{X}}} is an \eqn{m \times n} matrix whose columns are columns of
  observations on one of the \eqn{ n - 1} independent variables.  The first column
  contains \eqn{m} ones.  \eqn{{\bf{e}}} is an order \eqn{m} random vector of zero
  mean residual values.  \eqn{{\bf{\beta }}} is the order \eqn{n} vector of regression
  parameters.  The objective function that is minimized in the method of least squares is
  \eqn{\left( {{\bf{y}} - {\bf{X}}\;{\bf{\beta }}} \right)^\prime  \left( {{\bf{y}} - {\bf{X}}\;{\bf{\beta }}} \right)}.
  The solution to ths quadratic programming problem is
  \eqn{{\bf{\hat \beta }} = \left[ {\left( {{\bf{X'}}\;{\bf{X}}} \right)^{ - 1} \;{\bf{X'}}} \right]\;{\bf{y}}}
  The corresponding estimator for the residual vector is
  \eqn{{\bf{\hat e}} = {\bf{y}} - {\bf{X}}\;{\bf{\hat \beta }} = \left[ {{\bf{I}} - {\bf{X}}\;\left( {{\bf{X'}}\;{\bf{X}}} \right)^{ - 1} {\bf{X'}}} \right]{\bf{y}} = {\bf{M}}\;{\bf{y}}}.
  \eqn{{\bf{M}}} and \eqn{{{\bf{X}}\;\left( {{\bf{X'}}\;{\bf{X}}} \right)^{ - 1} {\bf{X'}}}} are idempotent.
  Idempotency of \eqn{{\bf{M}}} enters into the estimation of the variance of the estimator.
}
\value{
  A TRUE or FALSE value.
}
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.

  Chang, A. C., (1984). \emph{Fundamental Methods of Mathematical Economics},
  Third edition, McGraw-Hill.

  Green, W. H. (2003). \emph{Econometric Analysis}, Fifth edition, Prentice-Hall.
  
  Horn, R. A. and C. R. Johnson (1990). \emph{Matrix Analysis}, Cambridge University Press.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\examples{
A <- diag( 1, 3 )
is.idempotent.matrix( A )
B <- matrix( c( 1, 2, 3, 4 ), nrow=2, byrow=TRUE )
is.idempotent.matrix( B )
C <- matrix( c( 1, 0, 0, 0 ), nrow=2, byrow=TRUE )
is.idempotent.matrix( C )
}
\keyword{ math }

\name{stirling.matrix}
\alias{stirling.matrix}
\title{ Stirling Matrix }
\description{
  This function constructs and returns a Stirling matrix which is
  a lower triangular matrix containing the Stirling numbers of
  the second kind.
}
\usage{
stirling.matrix(n)
}
\arguments{
  \item{n}{  A positive integer value }
}
\details{
  The Stirling numbers of the second kind, \eqn{S_i^j}, are used
  in combinatorics to compute the number of ways a set of \eqn{i} objects
  can be partitioned into \eqn{j} non-empty subsets \eqn{j \le i}.  The numbers are also
  denoted by 
  \eqn{\left\{ {\begin{array}{c}i\\j\end{array}} \right\}}.  Stirling numbers of
  the second kind can be computed recursively with the equation
  \eqn{S_j^{i + 1} = S_{j - 1}^i + j\;S_j^i,\quad 1 \le i \le n - 1,\;1 \le j \le i}.
  The initial conditions for the recursion are
  \eqn{S_i^i = 1,\quad 0 \le i \le n} and
  \eqn{S_j^0 = S_0^j = 0,\quad 0 \le j \le n}.  The resultant numbers are organized
  in an order \eqn{n + 1} matrix
  \eqn{\left\lbrack {\begin{array}{ccccc}
{S_0^0}&0&0& \cdots &0\\
0&{S_1^1}&0& \cdots &0\\
0&{S_1^2}&{S_2^2}& \cdots &0\\
 \cdots & \cdots & \cdots & \cdots & \cdots \\
0&{S_1^n}&{S_2^n}& \cdots &{S_n^n}
\end{array}} \right\rbrack}.
}
\value{
  An order \eqn{n + 1} lower triangular matrix.
}
\references{
  Aceto, L. and D. Trigiante (2001). Matrices of Pascal and Other Greats,
  \emph{American Mathematical Monthly}, March 2001, 108(3), 232-245.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  If the argument n is not a positive integer, the function presents an error message and stops.
}
\examples{
S <- stirling.matrix( 10 )
print( S )
}
\keyword{ math }

\name{is.positive.semi.definite}
\alias{is.positive.semi.definite}
\title{ Test matrix for positive semi-definiteness }
\description{
  This function returns TRUE if the argument, a square symmetric real matrix x, is positive  semi-definite.  
}
\usage{
is.positive.semi.definite(x, tol=1e-8)
}
\arguments{
  \item{x}{ a matrix }
  \item{tol}{ a numeric tolerance level }
}
\details{
  For a positive semi-definite matrix, the eigenvalues should be non-negative. The R function \code{eigen}
  is used to compute the eigenvalues. If any of the eigenvalues is less than zero,
  then the matrix is not positive semi-definite.  Otherwise, the matrix is declared
  to be positive semi-definite.
}
\value{
  TRUE or FALSE.
}
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\seealso{
  \code{\link{is.positive.definite}}, 
  \code{\link{is.negative.definite}}, 
  \code{\link{is.negative.semi.definite}}, 
  \code{\link{is.indefinite}}
}
\examples{
###
### identity matrix is always positive definite
I <- diag( 1, 3 )
is.positive.semi.definite( I )
###
### positive definite matrix
### eigenvalues are 3.4142136 2.0000000 0.585786
###
A <- matrix( c( 2, -1, 0, -1, 2, -1, 0, -1, 2 ), nrow=3, byrow=TRUE )
is.positive.semi.definite( A )
###
### positive semi-defnite matrix
### eigenvalues are 4.732051 1.267949 8.881784e-16
###
B <- matrix( c( 2, -1, 2, -1, 2, -1, 2, -1, 2 ), nrow=3, byrow=TRUE )
is.positive.semi.definite( B )
###
### negative definite matrix
### eigenvalues are -0.5857864 -2.0000000 -3.4142136
###
C <- matrix( c( -2, 1, 0, 1, -2, 1, 0, 1, -2 ), nrow=3, byrow=TRUE )
is.positive.semi.definite( C )
###
### negative semi-definite matrix
### eigenvalues are 1.894210e-16 -1.267949 -4.732051
###
D <- matrix( c( -2, 1, -2, 1, -2, 1, -2, 1, -2 ), nrow=3, byrow=TRUE )
is.positive.semi.definite( D )
###
### indefinite matrix
### eigenvalues are 3.828427  1.000000 -1.828427
###
E <- matrix( c( 1, 2, 0, 2, 1, 2, 0, 2, 1 ), nrow=3, byrow=TRUE )
is.positive.semi.definite( E )
}
\keyword{ math }

\name{is.non.singular.matrix}
\alias{is.non.singular.matrix}
\title{ Test if matrix is non-singular }
\description{
  This function returns \code{TRUE} is the matrix argument is non-singular
  and \code{FALSE} otherwise.
}
\usage{
is.non.singular.matrix(x, tol = 1e-08)
}
\arguments{
  \item{x}{ a numeric square matrix }
  \item{tol}{ a numeric tolerance level usually left out }
}
\details{
  The determinant of the matrix \code{x} is first computed.
  If the absolute value of the determinant is greater than or equal to the given
  tolerance level, then a \code{TRUE} value is returned.
  Otherwise, a \code{FALSE} value is returned.
}
\value{
  TRUE or FALSE value.
}
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.

  Horn, R. A. and C. R. Johnson (1990). \emph{Matrix Analysis}, Cambridge University Press.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\seealso{
  \code{\link{is.singular.matrix}}
}
\examples{
A <- diag( 1, 3 )
is.non.singular.matrix( A )
B <- matrix( c( 0, 0, 3, 4 ), nrow=2, byrow=TRUE )
is.non.singular.matrix( B )
}
\keyword{ math }

\name{hadamard.prod}
\alias{hadamard.prod}
\title{ Hadamard product of two matrices }
\description{
  This function returns the Hadamard or Shur product of two matrices, x and y, that have the same row and
  column dimensions.
}
\usage{
hadamard.prod(x, y)
}
\arguments{
  \item{x}{ a numeric matrix or vector object }
  \item{y}{ a numeric matrix or vector object }
}
\details{
  The Hadamard product is an element-by-element product of the two matrices. Let \eqn{{\bf{x}}} 
  and \eqn{{\bf{x}}} be two \eqn{m \times n} numeric matrices.  The Hadamard product is \eqn{{\bf{x}}\, \circ \,{\bf{y}} = \left\lbrack {\begin{array}{cccc}
{{x_{1,1}}\,{y_{1,1}}}&{{x_{1,2}}\,{y_{1,2}}}& \cdots &{{x_{1,n}}\,{y_{1,n}}}\\
{{x_{2,1}}\,{y_{121}}}&{{x_{2,2}}\,{y_{2,2}}}& \cdots &{{x_{2,n}}\,{y_{2,n}}}\\
 \cdots & \cdots & \cdots & \cdots \\
{{x_{m,1}}\,{y_{m,1}}}&{{x_{m,2}}\,{y_{m,2}}}& \cdots &{{x_{m,n}}\,{y_{m,n}}}
\end{array}} \right\rbrack}.
  It uses the * operation in R.
}
\value{
  A matrix.
}
\references{
  Hadamard, J (1983). Resolution d'une question relative aux determinants, \emph{Bulletin des Sciences
  Mathematiques}, 17, 240-246.
  
  Styan, G. P. H. (1973). Hadamard Products and Multivariate Statistical Analysis,
  \emph{Linear Algebra and Its Applications}, Elsevier, 6, 217-240.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  The function converts vectors to matrices if necessary.
  The function stops running if x or y is not numeric and an error message is displayed.
  The function also stops running if x and y do not have the same row and column dimensions and an error mesage
  is displayed.
}
\examples{
x <- matrix( c( 1, 2, 3, 4 ), nrow=2, byrow=TRUE )
y <- matrix( c( 2, 4, 6, 8 ), nrow=2, byrow=TRUE )
z <- hadamard.prod( x, y )
print( z )
}
\keyword{ math }

\name{direct.prod}
\alias{direct.prod}
\title{ Direct prod of two arrays }
\description{
  This function computes the direct product of two arrays.  The arrays can be
  numerical vectors or matrices.  The result is a matrix.
}  
\usage{
direct.prod( x, y )
}
\arguments{
  \item{x}{ a numeric matrix or vector }
  \item{y}{ a numeric matrix or vector }
}
\details{
  If either \eqn{\bf{x}} or \eqn{\bf{y}} is a vector, it is converted to a matrix.
  Suppose that \eqn{\bf{x}} is an \eqn{m \times n} matrix and \eqn{\bf{y}} is an \eqn{ p \times q}
  matrix. Then, the function returns the matrix \eqn{\left\lbrack {\begin{array}{cccc}
{{x_{1,1}}\;{\bf{y}}}&{{x_{1,2}}\;{\bf{y}}}& \cdots &{{x_{1,n}}\;{\bf{y}}}\\
{{x_{2,1}}\;{\bf{y}}}&{{x_{2,2}}\;{\bf{y}}}& \cdots &{{x_{2,n}}\;{\bf{y}}}\\
 \cdots & \cdots & \cdots & \cdots \\
{{x_{m,1}}\;{\bf{y}}}&{{x_{m,2}}\;{\bf{y}}}& \cdots &{{x_{m,n}}\;{\bf{y}}}
\end{array}} \right\rbrack}.
}
\value{
  A numeric matrix.
}
\references{
  Magnus, J. R. and H. Neudecker (1999) \emph{Matrix Differential Calculus with Applications in Statistics and Econometrics},
  Second Edition, John Wiley.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu}, Kurt Hornik \email{Kurt.Hornik@wu-wien.ac.at} }
\examples{
x <- matrix( seq( 1, 4 ) )
y <- matrix( seq( 5, 8 ) )
print( direct.prod( x, y ) )
}
\keyword{ math }

\name{duplication.matrix}
\alias{duplication.matrix}
\title{ Duplication matrix for n by n matrices }
\description{
  This function returns a matrix with n * n rows and n * ( n + 1 ) / 2 columns
  that transforms vech(A) to vec(A) where A is a symmetric n by n matrix.
}
\usage{
duplication.matrix(n=1)
}
\arguments{
  \item{n}{ Row and column dimension }
}
\details{
  This function is a wrapper function for the function \code{D.matrix}.
  Let \eqn{{\bf{T}}_{i,j}} be an \eqn{n \times n} matrix with 1 in its \eqn{\left( {i,j} \right)} element \eqn{1 \le i,j \le n}.
  and zeroes elsewhere. These matrices are constructed by the function \code{T.matrices}.  The formula for the
  transpose of matrix \eqn{\bf{D}} is \eqn{{\bf{D'}} = \sum\limits_{j = 1}^n {\sum\limits_{i = j}^n {{{\bf{u}}_{i,j}}\;{{\left( {vec\;{{\bf{T}}_{i,j}}} \right)}^\prime }} } }
  where \eqn{{{{\bf{u}}_{i,j}}}} is the column vector in the order \eqn{\frac{1}{2}n\left( {n + 1} \right)} identity
  matrix for column \eqn{k = \left( {j - 1} \right)n + i - \frac{1}{2}j\left( {j - 1} \right)}.  The function
  \code{u.vectors} generates these vectors.
}  
\value{
  It returns an \eqn{{n^2}\; \times \;\frac{1}{2}n\left( {n + 1} \right)} matrix.
}
\references{
  Magnus, J. R. and H. Neudecker (1980). The elimination matrix, some lemmas and applications,
  \emph{SIAM Journal on Algebraic Discrete Methods}, 1(4), December 1980, 422-449.
  
  Magnus, J. R. and H. Neudecker (1999) \emph{Matrix Differential Calculus with Applications in Statistics and Econometrics},
  Second Edition, John Wiley.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu}, Kurt Hornik \email{Kurt.Hornik@wu-wien.ac.at} }
\seealso{
  \code{\link{D.matrix}},
  \code{\link{vec}},
  \code{\link{vech}}
}
\examples{
D <- duplication.matrix( 3 )
A <- matrix( c( 1, 2, 3,
                2, 3, 4,
                3, 4, 5), nrow=3, byrow=TRUE )
vecA <- vec( A )
vechA<- vech( A )
y <- D \%*\% vechA
print( y )
print( vecA )
}
\keyword{ math }

\name{H.matrices}
\alias{H.matrices}
\title{ List of H Matrices }
\description{
  This function constructs and returns a list of lists. The component of
  each sublist is derived from column vectors in an order r and order c identity matrix.
}
\usage{
H.matrices(r, c = r)
}
\arguments{
  \item{r}{ a positive integer value for an order r identity matrix }
  \item{c}{ a positive integer value for an order c identify matrix }
}
\details{
  Let \eqn{{{\bf{I}}_r} = \lbrack {\begin{array}{cccc}
  {{{\bf{a}}_1}}&{{{\bf{a}}_2}}& \cdots &{{{\bf{a}}_r}}
  \end{array}} \rbrack} be the order \eqn{r} identity matrix
  with corresponding unit vectors \eqn{{{{\bf{a}}_i}}} with one in
  its \eqn{i}th position and zeros elsewhere.  
  Let \eqn{{{\bf{I}}_c} = \lbrack {\begin{array}{cccc}
  {{{\bf{b}}_1}}&{{{\bf{b}}_2}}& \cdots &{{{\bf{b}}_c}}
  \end{array}} \rbrack} be the order \eqn{c} identity matrix
  with corresponding unit vectors \eqn{{{{\bf{b}}_i}}} with one in
  its \eqn{i}th position and zeros elsewhere.
  The \eqn{r \times c} matrix \eqn{{\bf{H}}{}_{i,j} = {{\bf{a}}_i}\;{{\bf{b'}}_j}}
  is used in the computation of the commutation matrix.
}
\value{
  A list with \eqn{r} components
  \item{1 }{A sublist of \eqn{c} components}
  \item{2 }{A sublist of \eqn{c} components}
 ...
  \item{r }{A sublist of c components}
  Each component \eqn{j} of sublist \eqn{i} is a matrix \eqn{{\bf{H}}_{i,j}}
}
\references{
  Magnus, J. R. and H. Neudecker (1979). The commutation matrix: some properties and
  applications, \emph{The Annals of Statistics}, 7(2), 381-394.
  
  Magnus, J. R. and H. Neudecker (1980). The elimination matrix, some lemmas and applications,
  \emph{SIAM Journal on Algebraic Discrete Methods}, 1(4), December 1980, 422-449.
  
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  The argument n must be an integer value greater than or equal to two.
}
\examples{
H.2.3 <- H.matrices( 2, 3 )
H.3 <- H.matrices( 3 )
}
\keyword{ math }

\name{N.matrix}
\alias{N.matrix}
\title{ Construct N Matrix }
\description{
  This function returns the order n square matrix that is the sum of
  an implicit commutation matrix and the order n identity matrix
  quantity divided by two
}
\usage{
N.matrix(n)
}
\arguments{
  \item{n}{ A positive integer matrix order }
}
\details{
  Let \eqn{{\bf{K}_n}} be the order \eqn{n} implicit commutation matrix (i.e., \eqn{{{\bf{K}}_{n,n}}} ).
  and \eqn{{{\bf{I}}_n}} the order \eqn{n} identity matrix.  The formula for the matrix is \eqn{{\bf{N}} = \frac{1}{2}\left( {{{\bf{K}}_n} + {{\bf{I}}_n}} \right)}.
}
\value{
  An order \eqn{n} matrix.
}
\references{
  Magnus, J. R. and H. Neudecker (1980). The elimination matrix, some lemmas and applications,
  \emph{SIAM Journal on Algebraic Discrete Methods}, 1(4), December 1980, 422-449.
  
  Magnus, J. R. and H. Neudecker (1999) \emph{Matrix Differential Calculus with Applications in Statistics and Econometrics},
  Second Edition, John Wiley.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  If the argument is not an integer, the function displays an error message and stops.
  If the argument is less than two, the function displays an error message and stops.
}
\seealso{
  \code{\link{K.matrix}}
}
\examples{
N <- N.matrix( 3 )
print( N )
}
\keyword{ math }

\name{frobenius.prod}
\alias{frobenius.prod}
\title{ Frobenius innter product of matrices }
\description{
  This function returns the Fronbenius inner product of two matrices, x and y, with the same row and column dimensions.
}
\usage{
frobenius.prod(x, y)
}
\arguments{
  \item{x}{ a numeric matrix or vector object }
  \item{y}{ a numeric matrix or vector object }
}
\details{
  The Frobenius inner product is the element-by-element sum of 
  the Hadamard or Shur product of two numeric matrices.  Let \eqn{{\bf{x}}} and
  \eqn{{\bf{y}}} be two \eqn{m \times n} matrices.  Then Frobenious inner product
  is computed as \eqn{\sum\limits_{i = 1}^m {\sum\limits_{j = 1}^n {x_{i,j} \;y_{i,j} } } }.
}
\value{
  A numeric value.
}
\references{
  Styan, G. P. H. (1973). Hadamard Products and Multivariate Statistical Analysis,
  \emph{Linear Algebra and Its Applications}, Elsevier, 6, 217-240.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\seealso{
  \code{\link{hadamard.prod}}
}
\note{
  The function converts vectors to matrices if necessary.
  The function stops running if x or y is not numeric and an error message is displayed.
  The function also stops running if x and y do not have the same row and column dimensions and an error mesage
  is displayed.
}
\examples{
x <- matrix( c( 1, 2, 3, 4 ), nrow=2, byrow=TRUE )
y <- matrix( c( 2, 4, 6, 8 ), nrow=2, byrow=TRUE )
z <- frobenius.prod( x, y )
print( z )
}
\keyword{ math }

\name{hankel.matrix}
\alias{hankel.matrix}
\title{ Hankel Matrix }
\description{
  This function constructs an order n Hankel matrix from the values in
  the order n vector x.  Each row of the matrix is a circular shift of
  the values in the previous row.
}
\usage{
hankel.matrix(n, x)
}
\arguments{
  \item{n}{ a positive integer value for order of matrix greater than 1 }
  \item{x}{ a vector of values used to construct the matrix }
}
\details{
  A Hankel matrix is a square matrix with constant skew diagonals.
  The determinant of a Hankel matrix is called a catalecticant.
  Hankel matrices are formed when the hidden Mark model is sought
  from a given sequence of data.
}
\value{
  An order \eqn{n} matrix.
}
\references{
  Power, S. C. (1982). \emph{Hankel Operators on Hilbert Spaces}, Research
  notes in mathematics, Series 64, Pitman Publishing.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  If the argument n is not a positive integer, the function presents an error message and stops.
  If the length of x is less than n, the function presents an error message and stops.
}
\examples{
H <- hankel.matrix( 4, seq( 1, 7 ) )
print( H )
}
\keyword{ math }

\name{spectral.norm}
\alias{spectral.norm}
\title{ Spectral norm of  matrix }
\description{
  This function returns the spectral norm of a real matrix.
}
\usage{
spectral.norm(x)
}
\arguments{
  \item{x}{ a numeric matrix or vector }
}
\details{
  Let \eqn{{\bf{x}}} be an \eqn{m \times n} real matrix.  The
  function computes the order \eqn{n} square matrixmatrix \eqn{{\bf{A}} = {\bf{x'}}\;{\bf{x}}}.
  The R function \code{eigen} is applied to this matrix to obtain the vector
  of eigenvalues \eqn{{\bf{\lambda }} = \left\lbrack {\begin{array}{cccc}
   {\lambda _1 } & {\lambda _2 } &  \cdots  & {\lambda _n }  \\
\end{array}} \right\rbrack}.  By construction the eigenvalues are in descending
  order of value so that the largest eigenvalue is \eqn{\lambda _1}.  Then
  the spectral norm is \eqn{\left\| {\bf{x}} \right\|_2  = \sqrt {\lambda _1 }}.
  If \eqn{{\bf{x}}} is a vector, then  \eqn{{\bf{L}}_2  = \sqrt {\bf{A}}} is returned.
}
\value{
  A numeric value.
}
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.

  Golub, G. H. and C. F. Van Loan (1996).  \emph{Matrix Computations}, Third Edition, The John
  Hopkins University Press.

  Horn, R. A. and C. R. Johnson (1985). \emph{Matrix Analysis}, Cambridge University Press.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  If the argument x is not numeric, an error message is displayed and the function terminates.
  If the argument is neither a matrix nor a vector, an error message is displayed and the 
  function terminates.
  If the product matrix \eqn{{\bf{x'}}\;{\bf{x}}} is negative definite, an error message
  displayed and the function terminates.
}
\examples{
x <- matrix( c( 2, 4, 2, 1, 3, 1, 5, 2, 1, 2, 3, 3 ), nrow=3, ncol=4, byrow=TRUE )
spectral.norm( x )
}
\keyword{ math }

\name{set.submatrix}
\alias{set.submatrix}
\title{ Store matrix inside another matrix }
\description{
  This function returns a matrix which is a copy of matrix x into which the contents of matrix y
  have been inserted at the given row and column.
}
\usage{
set.submatrix(x, y, row, col)
}
\arguments{
  \item{x}{ a matrix }
  \item{y}{ a matrix }
  \item{row}{ an integer row number }
  \item{col}{ an integer column number }
}
\value{
  A matrix.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  If the argument x is not a numeric matrix, then the function presents an error message and stops.
  If the argument y is not a numeric matrix, then the function presents an error message and stops.
  If the argument row is not a positive integer, then the function presents an error message and stops.
  If the argument col is not a positive integer, then the function presents an error message and stops.
  If the target row range does not overlap with the row range of argument x, then the function presents an error message and stops.
  If the target col range does not overlap with the col range of argument x, then the function presents an error message and stops.
}
\examples{
x <- matrix( seq( 1, 16, 1 ), nrow=4, byrow=TRUE )
y <- matrix( seq( 1, 4, 1 ), nrow=2, byrow=TRUE )
z <- set.submatrix( x, y, 3, 3 )
}
\keyword{ math }

\name{lu.decomposition}
\alias{lu.decomposition}
\title{ LU Decomposition of Square Matrix }
\description{
  This function performs an LU decomposition of the given square matrix argument
  the results are returned in a list of named components.  The Doolittle decomposition
  method is used to obtain the lower and upper triangular matrices
}
\usage{
lu.decomposition(x)
}
\arguments{
  \item{x}{ a numeric square matrix }
}
\details{
  The Doolittle decomposition without row exchanges is performed generating
  the lower and upper triangular matrices separately rather than in one matrix.
}
\value{
  A list with two named components.
  \item{L }{The numeric lower triangular matrix}
  \item{U }{The number upper triangular matrix}
}
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.

  Golub, G. H. and C. F. Van Loan (1996). \emph{Matrix Computations}, Third Edition,
  John Hopkins University Press
  
  Horn, R. A. and C. R. Johnson (1985). \emph{Matrix Analysis}, Cambridge University Press.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\examples{
A <- matrix( c ( 1, 2, 2, 1 ), nrow=2, byrow=TRUE)
luA <- lu.decomposition( A )
L <- luA$L
U <- luA$U
print( L )
print( U )
print( L \%*\% U )
print( A )
B <- matrix( c( 2, -1, -2, -4, 6, 3, -4, -2, 8 ), nrow=3, byrow=TRUE )
luB <- lu.decomposition( B )
L <- luB$L
U <- luB$U
print( L )
print( U )
print( L \%*\% U )
print( B )
}
\keyword{ math }

\name{T.matrices}
\alias{T.matrices}
\title{ List of T Matrices }
\description{
  This function constructs a list of lists.  The number of components in
  the high level list is n.  Each of the n components is also a list.
  Each sub-list has n components each of which is an order n square matrix.
}
\usage{
T.matrices(n)
}
\arguments{
  \item{n}{ a positive integer value for the order of the matrices }
}
\details{
  Let \eqn{{{\bf{E}}_{i,j}}\;i = 1, \ldots ,n\;;\;j = 1, \ldots ,n} 
  be a representative order \eqn{n} matrix created with function \code{E.matrices}.
  The order \eqn{n} matrix \eqn{{{\bf{T}}_{i,j}}} is defined as follows
  \eqn{{{\bf{T}}_{i,j}} = \left\{ {\begin{array}{cc}
{{{\bf{E}}_{i,j}}}&{i = j}\\
{{{\bf{E}}_{i,j}} + {{\bf{E}}_{j,i}}}&{i \ne j}
\end{array}} \right.}
}
\value{
  A list of \eqn{n} components.
  \item{1 }{A list of \eqn{n} components}
  \item{2 }{A list of \eqn{n} components}
 ...
  \item{n }{A list of \eqn{n} components}
  Each component \eqn{j} of sublist \eqn{i} is a matrix \eqn{{\bf{T}}_{i,j}}
}
\references{
  Magnus, J. R. and H. Neudecker (1980). The elimination matrix, some lemmas and applications,
  \emph{SIAM Journal on Algebraic Discrete Methods}, 1(4), December 1980, 422-449.
  
  Magnus, J. R. and H. Neudecker (1999) \emph{Matrix Differential Calculus with Applications in Statistics and Econometrics},
  Second Edition, John Wiley.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  The argument n must be an integer value greater than or equal to 2.
}
\seealso{
  \code{\link{E.matrices}}
}
\examples{
T <- T.matrices( 3 )
}
\keyword{ math }

\name{is.indefinite}
\alias{is.indefinite}
\title{ Test matrix for positive indefiniteness }
\description{
  This function returns TRUE if the argument, a square symmetric real matrix x, is indefinite.
  That is, the matrix has both positive and negative eigenvalues.
}
\usage{
is.indefinite(x, tol=1e-8)
}
\arguments{
  \item{x}{ a matrix }
  \item{tol}{ a numeric tolerance level }
}
\details{
  For an indefinite matrix, the matrix should positive and negative eigenvalues. The R function \code{eigen}
  is used to compute the eigenvalues. If any of the eigenvalues is absolute value is less than the
  given tolerance, that eigenvalue is replaced with zero. If the matrix has both positive and
  negative eigenvalues, it is declared to be indefinite.
}
\value{
  TRUE or FALSE.
}
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\seealso{
  \code{\link{is.positive.definite}}, 
  \code{\link{is.positive.semi.definite}}, 
  \code{\link{is.negative.definite}}, 
  \code{\link{is.negative.semi.definite}}
}
\examples{
###
### identity matrix is always positive definite
###
I <- diag( 1, 3 )
is.indefinite( I )
###
### positive definite matrix
### eigenvalues are 3.4142136 2.0000000 0.585786
###
A <- matrix( c( 2, -1, 0, -1, 2, -1, 0, -1, 2 ), nrow=3, byrow=TRUE )
is.indefinite( A )
###
### positive semi-defnite matrix
### eigenvalues are 4.732051 1.267949 8.881784e-16
###
B <- matrix( c( 2, -1, 2, -1, 2, -1, 2, -1, 2 ), nrow=3, byrow=TRUE )
is.indefinite( B )
###
### negative definite matrix
### eigenvalues are -0.5857864 -2.0000000 -3.4142136
###
C <- matrix( c( -2, 1, 0, 1, -2, 1, 0, 1, -2 ), nrow=3, byrow=TRUE )
is.indefinite( C )
###
### negative semi-definite matrix
### eigenvalues are 1.894210e-16 -1.267949 -4.732051
###
D <- matrix( c( -2, 1, -2, 1, -2, 1, -2, 1, -2 ), nrow=3, byrow=TRUE )
is.indefinite( D )
###
### indefinite matrix
### eigenvalues are 3.828427  1.000000 -1.828427
###
E <- matrix( c( 1, 2, 0, 2, 1, 2, 0, 2, 1 ), nrow=3, byrow=TRUE )
is.indefinite( E )
}
\keyword{ math }

\name{inf.norm}
\alias{inf.norm}
\title{ Compute the infinitity norm of a matrix }
\description{
  This function returns the \eqn{\left\| {\mathbf{x}} \right\|_\infty } norm of the matrix \eqn{{\mathbf{x}}}.
}
\usage{
inf.norm(x)
}
\arguments{
  \item{x}{  a numeric vector or matrix }
}
\details{
  Let \eqn{{\bf{x}}} be an \eqn{m \times n} numeric matrix.
  The formula used to compute the norm is 
  \eqn{\left\| {\bf{x}} \right\|_\infty   = \mathop {\max }\limits_{1 \le i \le m} \sum\limits_{j = 1}^n {\left| {x_{i,j} } \right|} }.
  This is merely the maximum absolute row sum of the \eqn{m \times n} maxtris.
}
\value{
  A numeric value.
}
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.

  Golub, G. H. and C. F. Van Loan (1996).  \emph{Matrix Computations}, Third Edition, The John
  Hopkins University Press.

  Horn, R. A. and C. R. Johnson (1985). \emph{Matrix Analysis}, Cambridge University Press.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\seealso{
  \code{\link{one.norm}}
}
\examples{
A <- matrix( c( 3, 5, 7, 2, 6, 4, 0, 2, 8 ), nrow=3, ncol=3, byrow=TRUE )
print( inf.norm( A ) )
}
\keyword{ math }

\name{upper.triangle}
\alias{upper.triangle}
\title{ Upper triangle portion of a matrix }
\description{
  Returns the lower triangle including the diagonal of a square numeric matrix.
}
\usage{
upper.triangle(x)
}
\arguments{
  \item{x}{ a matrix }
}
\value{
  A matrix.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.
}
\seealso{
  \code{\link{is.square.matrix}}
}
\examples{
A <- matrix( seq( 1, 9, 1 ), nrow=3, byrow=TRUE )
upper.triangle( A )
}
\keyword{ math }

\name{matrix.rank}
\alias{matrix.rank}
\title{ Rank of a square matrix }
\description{
  This function returns the rank of a square numeric matrix based on the selected method.
}
\usage{
matrix.rank(x, method = c("qr", "chol"))
}
\arguments{
  \item{x}{ a matrix }
  \item{method}{ a character string that specifies the method to be used }
}
\details{
  If the user specifies "qr" as the method, then the QR decomposition function is used to obtain the rank.
  If the user specifies "chol" as the method, the rank is obtained from the attributes of the value returned.
}
\value{
  An integer.
}
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  If the argument is not a square numeric matrix, then the function presents an error message and stops.
}
\seealso{
  \code{\link{is.square.matrix}}
}  
\examples{
A <- diag( seq( 1, 4, 1 ) )
matrix.rank( A )
B <- matrix( seq( 1, 16, 1 ), nrow=4, byrow=TRUE )
matrix.rank( B )
}
\keyword{ math }

\name{shift.down}
\alias{shift.down}
\title{ Shift matrix m rows down }
\description{
  This function returns a matrix that has had its rows shifted downwards filling the above rows
  with the given fill value.
}
\usage{
shift.down(A, rows = 1, fill = 0)
}
\arguments{
  \item{A}{ a matrix }
  \item{rows}{ the number of rows to be shifted }
  \item{fill}{ the fill value which as a default is zero }
}
\value{
  A matrix.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  If the argument A is not a numeric matrix, then the function presents an error message and stops.
  If the argument rows is not a positive integer, then the function presents an error message and stops.
}
\examples{
A <- matrix( seq( 1, 16, 1 ), nrow=4, byrow=TRUE )
shift.down( A, 1 )
shift.down( A, 3 )
}
\keyword{ math }

\name{is.negative.semi.definite}
\alias{is.negative.semi.definite}
\title{ Test matrix for negative semi definiteness }
\description{
  This function returns TRUE if the argument, a square symmetric real matrix x, is negative semi-negative.  
}
\usage{
is.negative.semi.definite(x, tol=1e-8)
}
\arguments{
  \item{x}{ a matrix }
  \item{tol}{ a numeric tolerance level }
}
\details{
  For a negative semi-definite matrix, the eigenvalues should be non-positive. 
  The R function \code{eigen} is used to compute the eigenvalues. 
  If any of the eigenvalues in absolute value is less than the given tolerance, that eigenvalue
  is replaced with zero.  Then, if any of the eigenvalues is greater than zero, the matrix
  is not negative semi-definite.  Otherwise, the matrix is declared to be negative semi-definite.
}
\value{
  TRUE or FALSE.
}
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\seealso{
  \code{\link{is.positive.definite}}, 
  \code{\link{is.positive.semi.definite}}, 
  \code{\link{is.negative.definite}}, 
  \code{\link{is.indefinite}}
}
\examples{
###
### identity matrix is always positive definite
I <- diag( 1, 3 )
is.negative.semi.definite( I )
###
### positive definite matrix
### eigenvalues are 3.4142136 2.0000000 0.585786
###
A <- matrix( c( 2, -1, 0, -1, 2, -1, 0, -1, 2 ), nrow=3, byrow=TRUE )
is.negative.semi.definite( A )
###
### positive semi-defnite matrix
### eigenvalues are 4.732051 1.267949 8.881784e-16
###
B <- matrix( c( 2, -1, 2, -1, 2, -1, 2, -1, 2 ), nrow=3, byrow=TRUE )
is.negative.semi.definite( B )
###
### negative definite matrix
### eigenvalues are -0.5857864 -2.0000000 -3.4142136
###
C <- matrix( c( -2, 1, 0, 1, -2, 1, 0, 1, -2 ), nrow=3, byrow=TRUE )
is.negative.semi.definite( C )
###
### negative semi-definite matrix
### eigenvalues are 1.894210e-16 -1.267949 -4.732051
###
D <- matrix( c( -2, 1, -2, 1, -2, 1, -2, 1, -2 ), nrow=3, byrow=TRUE )
is.negative.semi.definite( D )
###
### indefinite matrix
### eigenvalues are 3.828427  1.000000 -1.828427
###
E <- matrix( c( 1, 2, 0, 2, 1, 2, 0, 2, 1 ), nrow=3, byrow=TRUE )
is.negative.semi.definite( E )
}
\keyword{ math }

\name{entrywise.norm}
\alias{entrywise.norm}
\title{ Compute the entrywise norm of a matrix }
\description{
  This function returns the \eqn{\left\| {\bf{x}} \right\|_p } norm of the matrix \eqn{{\mathbf{x}}}.
}
\usage{
entrywise.norm(x,p)
}
\arguments{
  \item{x}{  a numeric vector or matrix }
  \item{p}{  a real value for the power }
}
\details{
  Let \eqn{{\bf{x}}} be an \eqn{m \times n} numeric matrix.
  The formula used to compute the norm is \eqn{\left\| {\bf{x}} \right\|_p  = \left( {\sum\limits_{i = 1}^m {\sum\limits_{j = 1}^n {\left| {x_{i,j} } \right|^p } } } \right)^{{1 \mathord{\left/
 {\vphantom {1 p}} \right.
 } p}}}.
}
\value{
  A numeric value.
}
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.

  Golub, G. H. and C. F. Van Loan (1996).  \emph{Matrix Computations}, Third Edition, The John
  Hopkins University Press.

  Horn, R. A. and C. R. Johnson (1985). \emph{Matrix Analysis}, Cambridge University Press.
}
\note{
  If argument x is not numeric, the function displays an error message and terminates.
  If argument x is neither a matrix nor a vector, the function displays an error message and terminates.
  If argument p is zero, the function displays an error message and terminates.
}  
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\seealso{
  \code{\link{one.norm}},
  \code{\link{inf.norm}}
}
\examples{
A <- matrix( c( 3, 5, 7, 2, 6, 4, 0, 2, 8 ), nrow=3, ncol=3, byrow=TRUE )
print( entrywise.norm( A, 2 ) )
}
\keyword{ math }

\name{is.skew.symmetric.matrix}
\alias{is.skew.symmetric.matrix}
\title{ Test for a skew-symmetric matrix }
\description{
  This function returns \code{TRUE} if the matrix argument x is
  a skew symmetric matrix, i.e., the transpose of the matrix is
  the negative of the matrix.  Otherwise, \code{FALSE} is returned.
}
\usage{
is.skew.symmetric.matrix(x, tol = 1e-08)
}
\arguments{
  \item{x}{ a numeric square matrix }
  \item{tol}{ a numeric tolerance level usually left out }
}
\details{
  Let \eqn{{\bf{x}}} be an order \eqn{n} matrix.  If every element
  of the matrix \eqn{{\bf{x}} + {\bf{x'}}} in absolute value is less
  than the given tolerance, then the matrix argument is declared to be
  skew symmetric.
}
\value{
  A TRUE or FALSE value.
}
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.

  Horn, R. A. and C. R. Johnson (1990). \emph{Matrix Analysis}, Cambridge University Press.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\examples{
A <- diag( 1, 3 )
is.skew.symmetric.matrix( A )
B <- matrix( c( 0, -2, -1, -2, 0, -4, 1, 4, 0 ), nrow=3, byrow=TRUE )
is.skew.symmetric.matrix( B )
C <- matrix( c( 0, 2, 1, 2, 0, 4, 1, 4, 0 ), nrow=3, byrow=TRUE )
is.skew.symmetric.matrix( C )
}
\keyword{ math }

\name{matrix.trace}
\alias{matrix.trace}
\title{ The trace of a matrix }
\description{
  This function returns the trace of a given square numeric matrix.
}
\usage{
matrix.trace(x)
}
\arguments{
  \item{x}{ a matrix }
}
\value{
  A numeric value which is the sum of the values on the diagonal.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.
}
\note{
  If the argument x is not numeric, the function presents and error message and terminates.
  If the argument x is not a square matrix, the function presents an error message and terminates.
}
\examples{
A <- matrix( seq( 1, 16, 1 ), nrow=4, byrow=TRUE )
matrix.trace( A )
}
\keyword{ math }

\name{matrix.power}
\alias{matrix.power}
\title{ Matrix Raised to a Power }
\description{
  This function computes the k-th power of order n square matrix x
  If k is zero, the order n identity matrix is returned.  argument k
  must be an integer.
}
\usage{
matrix.power(x, k)
}
\arguments{
  \item{x}{ a numeric square matrix }
  \item{k}{ a numeric exponent }
}
\details{
  The matrix power is computed by successive matrix multiplications.  If the
  exponent is zero, the order n identity matrix is returned.  If the exponent
  is negative, the inverse of the matrix is raised to the given power.
}
\value{
  An order \eqn{n} matrix.
}
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\examples{
A <- matrix( c ( 1, 2, 2, 1 ), nrow=2, byrow=TRUE)
matrix.power( A, -2 )
matrix.power( A, -1 )
matrix.power( A, 0 )
matrix.power( A, 1 )
matrix.power( A, 2 )
}
\keyword{ math }

\name{is.negative.definite}
\alias{is.negative.definite}
\title{ Test matrix for negative definiteness }
\description{
  This function returns TRUE if the argument, a square symmetric real matrix x, is negative definite.  
}
\usage{
is.negative.definite(x, tol=1e-8)
}
\arguments{
  \item{x}{ a matrix }
  \item{tol}{ a numeric tolerance level }
}
\details{
  For a negative definite matrix, the eigenvalues should be negative. The R function \code{eigen}
  is used to compute the eigenvalues. If any of the eigenvalues in absolute value is less than
  the given tolerance, that eigenvalue is replaced with zero. If any of the eigenvalues is greater than or equal to zero,
  then the matrix is not negative definite.   Otherwise, the matrix is declared to be negative definite.
}
\value{
  TRUE or FALSE.
}
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\seealso{
  \code{\link{is.positive.definite}}, 
  \code{\link{is.positive.semi.definite}}, 
  \code{\link{is.negative.semi.definite}}, 
  \code{\link{is.indefinite}}
}
\examples{
###
### identity matrix is always positive definite
I <- diag( 1, 3 )
is.negative.definite( I )
###
### positive definite matrix
### eigenvalues are 3.4142136 2.0000000 0.585786
###
A <- matrix( c( 2, -1, 0, -1, 2, -1, 0, -1, 2 ), nrow=3, byrow=TRUE )
is.negative.definite( A )
###
### positive semi-defnite matrix
### eigenvalues are 4.732051 1.267949 8.881784e-16
###
B <- matrix( c( 2, -1, 2, -1, 2, -1, 2, -1, 2 ), nrow=3, byrow=TRUE )
is.negative.definite( B )
###
### negative definite matrix
### eigenvalues are -0.5857864 -2.0000000 -3.4142136
###
C <- matrix( c( -2, 1, 0, 1, -2, 1, 0, 1, -2 ), nrow=3, byrow=TRUE )
is.negative.definite( C )
###
### negative semi-definite matrix
### eigenvalues are 1.894210e-16 -1.267949 -4.732051
###
D <- matrix( c( -2, 1, -2, 1, -2, 1, -2, 1, -2 ), nrow=3, byrow=TRUE )
is.negative.definite( D )
###
### indefinite matrix
### eigenvalues are 3.828427  1.000000 -1.828427
###
E <- matrix( c( 1, 2, 0, 2, 1, 2, 0, 2, 1 ), nrow=3, byrow=TRUE )
is.negative.definite( E )
}
\keyword{ math }

\name{is.square.matrix}
\alias{is.square.matrix}
\title{ Test for square matrix }
\description{
  The function returns TRUE if the argument is a square matrix and FALSE otherwise.
}
\usage{
is.square.matrix(x)
}
\arguments{
  \item{x}{ a matrix }
}
\value{
  TRUE or FALSE
}
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\examples{
A <- matrix( seq( 1, 12, 1 ), nrow=3, byrow=TRUE )
is.square.matrix( A )
B <- matrix( seq( 1, 16, 1 ), nrow=4, byrow=TRUE )
is.square.matrix( B )
}
\keyword{ math }

\name{u.vectors}
\alias{u.vectors}
\title{ u vectors of an identity matrix }
\description{
  This function constructs an order n * ( n + 1 ) / 2 identity matrix
  and an order matrix u that that maps the ordered pair of indices
  (i,j) i=j, ..., n; j=1, ..., n to a column in this identity matrix.
}
\usage{
u.vectors(n)
}
\arguments{
  \item{n}{ a positive integer value for the order of underlying matrices }
}
\details{
  The function firsts constructs an identity matrix of order \eqn{\frac{1}{2}n\left( {n + 1} \right)}.
  \eqn{{{{\bf{u}}_{i,j}}}} is the column vector in the order \eqn{\frac{1}{2}n\left( {n + 1} \right)} identity
  matrix for column \eqn{k = \left( {j - 1} \right)n + i - \frac{1}{2}j\left( {j - 1} \right)}.
}
\value{
  A list with two named components
  \item{k }{order \eqn{n} square matrix that maps each ordered pair (i,j) to a column in the identity matrix}
  \item{I }{order \eqn{\frac{1}{2}n\left( {n + 1} \right)} identity matrix}
}
\references{
  Magnus, J. R. and H. Neudecker (1980). The elimination matrix, some lemmas and applications,
  \emph{SIAM Journal on Algebraic Discrete Methods}, 1(4), December 1980, 422-449.
  
  Magnus, J. R. and H. Neudecker (1999) \emph{Matrix Differential Calculus with Applications in Statistics and Econometrics},
  Second Edition, John Wiley.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  If the argument is not an integer, the function displays an error message and stops.
  If the argument is less than two, the function displays an error message and stops.
}
\examples{
u <- u.vectors( 3 )
}
\keyword{ math }

\name{matrix.inverse}
\alias{matrix.inverse}
\title{ Inverse of a square matrix }
\description{
  This function returns the inverse of a square matrix computed using the R function solve.
}
\usage{
matrix.inverse(x)
}
\arguments{
  \item{x}{ a square numeric matrix }
}
\value{
  A matrix.
}
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\examples{
A <- matrix( c ( 1, 2, 2, 1 ), nrow=2, byrow=TRUE)
print( A )
invA <- matrix.inverse( A )
print( invA )
print( A \%*\% invA )
print( invA \%*\% A )
}
\keyword{ math }

\name{is.positive.definite}
\alias{is.positive.definite}
\title{ Test matrix for positive definiteness }
\description{
  This function returns TRUE if the argument, a square symmetric real matrix x, is positive definite.  
}
\usage{
is.positive.definite(x, tol=1e-8)
}
\arguments{
  \item{x}{ a matrix }
  \item{tol}{ a numeric tolerance level }
}
\details{
  For a positive definite matrix, the eigenvalues should be positive. The R function \code{eigen}
  is used to compute the eigenvalues. If any of the eigenvalues in absolute value is less than the
  given tolerance, that eigenvalue is replaced with zero. If any of the eigenvalues is less than or equal to zero,
  then the matrix is not positive definite.  Otherwise, the matrix is declared to be positive definite.
}
\value{
  TRUE or FALSE.
}
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\seealso{
  \code{\link{is.positive.semi.definite}}, 
  \code{\link{is.negative.definite}}, 
  \code{\link{is.negative.semi.definite}}, 
  \code{\link{is.indefinite}}
}
\examples{
###
### identity matrix is always positive definite
I <- diag( 1, 3 )
is.positive.definite( I )
###
### positive definite matrix
### eigenvalues are 3.4142136 2.0000000 0.585786
###
A <- matrix( c( 2, -1, 0, -1, 2, -1, 0, -1, 2 ), nrow=3, byrow=TRUE )
is.positive.definite( A )
###
### positive semi-defnite matrix
### eigenvalues are 4.732051 1.267949 8.881784e-16
###
B <- matrix( c( 2, -1, 2, -1, 2, -1, 2, -1, 2 ), nrow=3, byrow=TRUE )
is.positive.definite( B )
###
### negative definite matrix
### eigenvalues are -0.5857864 -2.0000000 -3.4142136
###
C <- matrix( c( -2, 1, 0, 1, -2, 1, 0, 1, -2 ), nrow=3, byrow=TRUE )
is.positive.definite( C )
###
### negative semi-definite matrix
### eigenvalues are 1.894210e-16 -1.267949 -4.732051
###
D <- matrix( c( -2, 1, -2, 1, -2, 1, -2, 1, -2 ), nrow=3, byrow=TRUE )
is.positive.definite( D )
###
### indefinite matrix
### eigenvalues are 3.828427  1.000000 -1.828427
###
E <- matrix( c( 1, 2, 0, 2, 1, 2, 0, 2, 1 ), nrow=3, byrow=TRUE )
is.positive.definite( E )
}
\keyword{ math }

\name{symmetric.pascal.matrix}
\alias{symmetric.pascal.matrix}
\title{ Symmetric Pascal matrix }
\description{
  This function returns an n by n symmetric Pascal matrix.
}
\usage{
symmetric.pascal.matrix(n)
}
\arguments{
  \item{n}{ Order of the matrix }
}
\details{
  In mathematics, particularly matrix theory and combinatorics, the symmetric Pascal matrix is a square matrix
  from which you can derive binomial coefficients.  The matrix is an order \eqn{n} symmetric
  matrix with typical element given by \eqn{{S_{i,j}} = {{n!} \mathord{\left/
 {\vphantom {{n!} {\left[ {r!\;\left( {n - r} \right)!} \right]}}} \right.
 } {\left[ {r!\;\left( {n - r} \right)!} \right]}}} where
  \eqn{n = i + j - 2} and \eqn{r = i - 1}.  The binomial coefficients are elegantly recovered from the
  symmetric Pascal matrix by performing an \eqn{LU} decomposition as \eqn{{\bf{S}} = {\bf{L}}\;{\bf{U}}}.
}  
\value{
  An order \eqn{n} matrix.
}
\references{
  Call, G. S. and D. J. Velleman, (1993). Pascal's matrices, \emph{American Mathematical Monthly},
  April 1993, 100, 372-376.
  
  Edelman, A. and G. Strang, (2004). Pascal Matrices, \emph{American Mathematical Monthly},
  111(3), 361-385.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  If the argument n is not a positive integer, the function presents an error message and stops.
}
\examples{
S <- symmetric.pascal.matrix( 4 )
print( S )
}
\keyword{ math }

\name{elimination.matrix}
\alias{elimination.matrix}
\title{ Elimination matrix for lower triangular matrices }
\description{
  This function returns a matrix with n * ( n + 1 ) / 2 rows and N * n columns which
  for any lower triangular matrix A transforms vec( A ) into vech(A)
}
\usage{
elimination.matrix(n)
}
\arguments{
  \item{n}{ row or column dimension }
}
\details{
  This function is a wrapper function to the function \code{L.matrix}.
  The formula used to compute the L matrix which is also called the elimination matrix is  \eqn{{\bf{L}} = \sum\limits_{j = 1}^n {\sum\limits_{i = j}^n {{{\bf{u}}_{i,j}}{{\left( {vec\;{{\bf{E}}_{i,j}}} \right)}^\prime }} } }
  \eqn{{{{\bf{u}}_{i,j}}}} are the order \eqn{n\left( {n + 1} \right)/2} vectors constructed by the function \code{u.vectors}.
  \eqn{{{{\bf{E}}_{i,j}}}} are the \eqn{ n \times n} matrices constructed by the function \code{E.matrices}.
}  
\value{
  An \eqn{\left[ {\frac{1}{2}n\left( {n + 1} \right)} \right] \times {n^2}} matrix.
}
\references{
  Magnus, J. R. and H. Neudecker (1980). The elimination matrix, some lemmas and applications,
  \emph{SIAM Journal on Algebraic Discrete Methods}, 1(4), December 1980, 422-449.
  
  Magnus, J. R. and H. Neudecker (1999) \emph{Matrix Differential Calculus with Applications in Statistics and Econometrics},
  Second Edition, John Wiley.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  If the argument is not an integer, the function displays an error message and stops.
  If the argument is less than two, the function displays an error message and stops.
}
\seealso{
  \code{\link{E.matrices}},
  \code{\link{L.matrix}},
  \code{\link{u.vectors}}
}
\examples{
L <- elimination.matrix( 4 )
A <- lower.triangle( matrix( seq( 1, 16, 1 ), nrow=4, byrow=TRUE ) )
vecA <- vec( A )
vechA <- vech( A )
y <- L \%*\% vecA
print( y )
print( vechA )
}
\keyword{ math }

\name{shift.right}
\alias{shift.right}
\title{Shift matrix n columns to the right }
\description{
  This function returns a matrix that has been shifted to the right n columns
  filling the previous columns with the given fill value.
}
\usage{
shift.right(A, cols = 1, fill = 0)
}
\arguments{
  \item{A}{ a matrix }
  \item{cols}{ integer number of columns to be shifted to the right }
  \item{fill}{ the fill which as default value zero }
}
\value{
  A matrix.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  If the argument A is not a numeric matrix, then the function presents an error message and stops.
  If the argument rows is not a positive integer, then the function presents an error message and stops.
}
\examples{
A <- matrix( seq( 1, 16, 1 ), nrow=4, byrow=TRUE )
shift.right( A, 1 )
shift.right( A, 2 )
}
\keyword{ math }

\name{commutation.matrix}
\alias{commutation.matrix}
\title{ Commutation matrix for r by c numeric matrices }
\description{
  This function returns a square matrix of order p = r * c that, 
  for an r by c matrix A, transforms
  vec(A) to vec(A') where prime denotes transpose.
}
\usage{
commutation.matrix(r, c=r)
}
\arguments{
  \item{r}{ a positive integer integer row dimension }
  \item{c}{ a positive integer integer column dimension }
}
\details{
  This function is a wrapper function that uses the function \code{K.matrix} to do the actual work.
  The \eqn{r \times c} matrices \eqn{{\bf{H}}{}_{i,j}} constructed
  by the function \code{H.matrices} are combined using direct product
  to generate the commutation product with the following formula
  \eqn{{{\bf{K}}_{r,c}} = \sum\limits_{i = 1}^r {\sum\limits_{j = 1}^c {\left( {{{\bf{H}}_{i,j}} \otimes {{{\bf{H'}}}_{i,j}}} \right)} }}
}
\value{
  An order \eqn{\left( {r\;c} \right)} matrix.
}
\references{
  Magnus, J. R. and H. Neudecker (1979). The commutation matrix: some properties and applications,
  \emph{The Annals of Statistics}, 7(2), 381-394.
  
  Magnus, J. R. and H. Neudecker (1999) \emph{Matrix Differential Calculus with Applications in Statistics and Econometrics},
  Second Edition, John Wiley.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  If either argument is less than 2, then the function stops and displays an appropriate error mesage.
  If either argument is not an integer, then the function stops and displays an appropriate error mesage
}
\seealso{
  \code{\link{H.matrices}},
  \code{\link{K.matrix}}
}
\examples{
K <- commutation.matrix( 3, 4 )
A <- matrix( seq( 1, 12, 1 ), nrow=3, byrow=TRUE )
vecA <- vec( A )
vecAt <- vec( t( A ) )
print( K \%*\% vecA )
print( vecAt )
}
\keyword{ math }

\name{E.matrices}
\alias{E.matrices}
\title{ List of E Matrices }
\description{
  This function constructs and returns a list of lists. The component of
  each sublist is a square matrix derived from the column vectors of
  an order n identity matrix.
}
\usage{
E.matrices(n)
}
\arguments{
  \item{n}{ a positive integer for the order of the identity matrix
}
}
\details{
  Let \eqn{{{\bf{I}}_n} = \lbrack {\begin{array}{cccc}
  {{{\bf{e}}_1}}&{{{\bf{e}}_2}}& \cdots &{{{\bf{e}}_n}}
  \end{array}} \rbrack}{} be the order \eqn{n} identity matrix
  with corresponding unit vectors \eqn{{{{\bf{e}}_i}}} with one in
  its \eqn{i}th position and zeros elsewhere.  
  The \eqn{n \times n} matrix \eqn{{{\bf{E}}_{i,j}}} is computed
  from the unit vectors \eqn{{{{\bf{e}}_i}}} and \eqn{{{{\bf{e}}_j}}}
  as \eqn{{{\bf{E}}_{i,j}} = {{\bf{e}}_i}\;{{\bf{e'}}_j}}.  These matrices
  are stored as components in a list of lists.
}
\value{
  A list with \eqn{n} components
  \item{1 }{A sublist of \eqn{n} components}
  \item{2 }{A sublist of \eqn{n} components}
 ...
  \item{n }{A sublist of \eqn{n} components}
  Each component \eqn{j} of sublist \eqn{i} is a matrix \eqn{{\bf{E}}_{i,j}}
}
\references{
  Magnus, J. R. and H. Neudecker (1980). The elimination matrix, some lemmas and applications,
  \emph{SIAM Journal on Algebraic Discrete Methods}, 1(4), December 1980, 422-449.
  
  Magnus, J. R. and H. Neudecker (1999). \emph{Matrix Differential Calculus with Applications in Statistics and Econometrics},
  Second Edition, John Wiley.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  The argument n must be an integer value greater than or equal to 2.
}
\examples{
E <- E.matrices( 3 )
}
\keyword{ math }

\name{svd.inverse}
\alias{svd.inverse}
\title{ SVD Inverse of a square matrix }
\description{
  This function returns the inverse of a matrix using singular value
  decomposition.  If the matrix is a square matrix, this should be equivalent
  to using the \code{solve} function.  If the matrix is not a square matrix,
  then the result is the Moore-Penrose pseudo inverse.
}
\usage{
svd.inverse(x)
}
\arguments{
  \item{x}{ a numeric matrix }
}
\value{
  A matrix.
}
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\examples{
A <- matrix( c ( 1, 2, 2, 1 ), nrow=2, byrow=TRUE)
invA <- svd.inverse( A )
print( A )
print( invA )
print( A \%*\% invA )
B <- matrix( c( -1, 2, 2 ), nrow=1, byrow=TRUE )
invB <- svd.inverse( B )
print( B )
print( invB )
print( B \%*\% invB )
}
\keyword{ math }

\name{is.diagonal.matrix}
\alias{is.diagonal.matrix}
\title{ Test for diagonal square matrix }
\description{
  This function returns \code{TRUE} if the given matrix argument x
  is a square numeric matrix and that the off-diagonal elements are close
  to zero in absolute value to within the given tolerance level. Otherwise,
  a \code{FALSE} value is returned.
}
\usage{
is.diagonal.matrix(x, tol = 1e-08)
}
\arguments{
  \item{x}{ a numeric square matrix }
  \item{tol}{ a numeric tolerance level usually left out }
}
\value{
  A TRUE or FALSE value.
}
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.

  Horn, R. A. and C. R. Johnson (1990). \emph{Matrix Analysis}, Cambridge University Press.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\examples{
A <- diag( 1, 3 )
is.diagonal.matrix( A )
B <- matrix( c( 1, 2, 3, 4 ), nrow=2, byrow=TRUE )
is.diagonal.matrix( B )
C <- matrix( c( 1, 0, 0, 0 ), nrow=2, byrow=TRUE )
is.diagonal.matrix( C )
}
\keyword{ math }

\name{pascal.matrix}
\alias{pascal.matrix}
\title{ Pascal matrix }
\description{
  This function returns an n by n Pascal matrix.
}
\usage{
pascal.matrix(n)
}
\arguments{
  \item{n}{ Order of the matrix }
}
\details{
  In mathematics, particularly matrix theory and combinatorics, the Pascal matrix is a lower triangular matrix
  with binomial coefficients in the rows.  It is easily obtained by performing an LU decomposition on 
  the symmetric Pascal matrix of the same order and returning the lower triangular matrix.
}  
\value{
  An order \eqn{n} matrix.
}
\references{
  Aceto, L. and D. Trigiante, (2001). Matrices of Pascal and Other Greats, \emph{American
  Mathematical Monthly}, March 2001, 232-245.

  Call, G. S. and D. J. Velleman, (1993). Pascal's matrices, \emph{American Mathematical Monthly},
  April 1993, 100, 372-376.
  
  Edelman, A. and G. Strang, (2004). Pascal Matrices, \emph{American Mathematical Monthly},
  111(3), 361-385.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  If the argument n is not a positive integer, the function presents an error message and stops.
}
\seealso{
  \code{\link{lu.decomposition}},
  \code{\link{symmetric.pascal.matrix}}
}
\examples{
P <- pascal.matrix( 4 )
print( P )
}
\keyword{ math }

\name{shift.up}
\alias{shift.up}
\title{ Shift matrix m rows up }
\description{
  This function returns a matrix where the argument as been shifted up the given number of rows filling
  the bottom rows with the given fill value.
}
\usage{
shift.up(A, rows = 1, fill = 0)
}
\arguments{
  \item{A}{ a matrix }
  \item{rows}{ integer number of rows}
  \item{fill}{ fill value which as the default value of zero }
}
\value{
  A matrix.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  If the argument A is not a numeric matrix, then the function presents an error message and stops.
  If the argument rows is not a positive integer, then the function presents an error message and stops.
}
\examples{
A <- matrix( seq( 1, 16, 1 ), nrow=4, byrow=TRUE )
shift.up( A, 1 )
shift.up( A, 3 )
}
\keyword{ math }

\name{hilbert.matrix}
\alias{hilbert.matrix}
\title{ Hilbert matrices }
\description{
  This function returns an n by n Hilbert matrix.
}
\usage{
hilbert.matrix(n)
}
\arguments{
  \item{n}{ Order of the Hilbert matrix }
}
\details{
  A Hilbert matrix is an order \eqn{n} square matrix of unit fractions with elements 
  defined as \eqn{H_{i,j}  = {1 \mathord{\left/
 {\vphantom {1 {\left( {i + j - 1} \right)}}} \right.
 } {\left( {i + j - 1} \right)}}}.
}
\value{
  A matrix.
}
\references{
  Hilbert, David (1894). Ein Beitrag zur Theorie des Legendre schen Polynoms,
  \emph{Acta Mathematica}, Springer, Netherlands, 18, 155-159.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  If the argument is less than or equal to zero, the function displays an error message and stops.
  If the argument is not an integer, the function displays an error message and stops.
}
\examples{
H <- hilbert.matrix( 4 )
print( H )
}
\keyword{ math }

\name{vec}
\alias{vec}
\title{ Vectorize a matrix }
\description{
  This function returns a column vector that is a stack of the columns of x, an m by n matrix.
}
\usage{
vec(x)
}
\arguments{
  \item{x}{ a matrix }
}
\value{
  A matrix with \eqn{m\;n} rows and one column.
}
\references{
  Magnus, J. R. and H. Neudecker (1999) \emph{Matrix Differential Calculus with Applications in Statistics and Econometrics},
  Second Edition, John Wiley.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\examples{
x <- matrix( seq( 1, 16, 1 ), nrow=4, byrow=TRUE )
print( x )
vecx <- vec( x )
print( vecx )
}
\keyword{ math }

\name{shift.left}
\alias{shift.left}
\title{ Shift a matrix n columns to the left }
\description{
  This function returns a matrix that has been shifted n columns to the left
  filling the subsqeuent columns with the given fill value
}
\usage{
shift.left(A, cols = 1, fill = 0)
}
\arguments{
  \item{A}{ a matrix }
  \item{cols}{ integer number of columns to be shifted to the left }
  \item{fill}{ the fill value which as as a default zero }
}
\value{
  A matrix.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  If the argument A is not a numeric matrix, then the function presents an error message and stops.
  If the argument cols is not a positive integer, then the function presents an error message and stops.
}
\examples{
A <- matrix( seq( 1, 12, 1 ), nrow=3, byrow=TRUE )
shift.left( A, 1 )
shift.left( A, 2 )
}
\keyword{ math }

\name{direct.sum}
\alias{direct.sum}
\title{ Direct sum of two arrays }
\description{
  This function computes the direct sum of two arrays.  The arrays can be
  numerical vectors or matrices.  The result ia the block diagonal matrix.
}  
\usage{
direct.sum( x, y )
}
\arguments{
  \item{x}{ a numeric matrix or vector }
  \item{y}{ a numeric matrix or vector }
}
\value{
  A numeric matrix.
}
\details{
  If either \eqn{\bf{x}} or y is a vector, it is converted to a matrix.  The result
  is a block diagonal matrix \eqn{\left\lbrack {\begin{array}{cc}
   {\bf{x}} & {\bf{0}}  \\
   {\bf{0}} & {\bf{y}}  \\
\end{array}} \right\rbrack}.
}
\references{
  Magnus, J. R. and H. Neudecker (1999) \emph{Matrix Differential Calculus with Applications in Statistics and Econometrics},
  Second Edition, John Wiley.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu}, Kurt Hornik \email{Kurt.Hornik@wu-wien.ac.at} }
\examples{
x <- matrix( seq( 1, 4 ) )
y <- matrix( seq( 5, 8 ) )
print( direct.sum( x, y ) )
}
\keyword{ math }

\name{is.symmetric.matrix}
\alias{is.symmetric.matrix}
\title{ Test for symmetric numeric matrix }
\description{
  This function returns TRUE if the argument is a numeric symmetric square matrix and FALSE otherwise.
}
\usage{
is.symmetric.matrix(x)
}
\arguments{
  \item{x}{ an R object }
}
\value{
  TRUE or FALSE.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\references{
  Bellman, R. (1987). \emph{Matrix Analysis}, Second edition, Classics in Applied Mathematics,
  Society for Industrial and Applied Mathematics.
}
\note{ 
  If the argument is not a numeric matrix, the function displays an error message and stops.
  If the argument is not a square matrix, the function displays an error message and stops.
}
\seealso{
  \code{\link{is.square.matrix}}
}
\examples{
A <- matrix( c( 1, 2, 3, 4 ), nrow=2, byrow=TRUE )
is.symmetric.matrix( A )
B <- matrix( c( 1, 2, 2, 1 ), nrow=2, byrow=TRUE )
is.symmetric.matrix( B )
}
\keyword{ math }

\name{toeplitz.matrix}
\alias{toeplitz.matrix}
\title{ Toeplitz Matrix }
\description{
  This function constructs an order n Toeplitz matrix from the values in
  the order 2 * n - 1 vector x.
}
\usage{
toeplitz.matrix(n, x)
}
\arguments{
  \item{n}{ a positive integer value for order of matrix greater than 1 }
  \item{x}{ a vector of values used to construct the matrix }
}
\details{
  The element \code{T[i,j]} in the Toeplitz matrix is \code{x[i-j+n]}.
}
\value{
  An order n matrix.
}
\references{
  Monahan, J. F. (2011). \emph{Numerical Methods of Statistics}, Cambridge
  University Press.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  If the argument n is not a positive integer, the function presents an error message and stops.
  If the length of x is not equal to 2 * n - 1, the function presents an error message and stops.
}
\examples{
T <- toeplitz.matrix( 4, seq( 1, 7 ) )
print( T )
}
\keyword{ math }

\name{frobenius.matrix}
\alias{frobenius.matrix}
\title{ Frobenius Matrix }
\description{
  This function returns an order n Frobenius matrix that is useful
  in numerical mathematics.
}
\usage{
frobenius.matrix(n)
}
\arguments{
  \item{n}{  a positive integer value greater than 1}
}
\details{
  The Frobenius matrix is also called the companion matrix.  It arises
  in the solution of systems of linear first order differential equations.
  The formula for the order \eqn{n} Frobenius matrix is \eqn{{\bf{F}} = 
\left\lbrack {\begin{array}{ccccc}0&0& \cdots &0&{{{\left( { - 1} \right)}^{n - 1}}
\left( {\begin{array}{ccccc}n\\0\end{array}} \right)}\\1&0& \cdots &0&{{{\left( { - 1} \right)}^{n - 2}}
\left( {\begin{array}{ccccc}n\\1\end{array}} \right)}\\0&1& \ddots &0&{{{\left( { - 1} \right)}^{n - 3}}
\left( {\begin{array}{ccccc}n\\2\end{array}} \right)}\\ \vdots & \vdots & \ddots & \vdots & \vdots \\0&0& \cdots &1&{{{\left( { - 1} \right)}^0}
\left( {\begin{array}{ccccc}n\\{n - 1}\end{array}} 
\right)}\end{array}} 
\right\rbrack}.
}
\value{
  An order \eqn{n} matrix
}
\references{
  Aceto, L. and D. Trigiante (2001). Matrices of Pascal and Other Greats,
  \emph{American Mathematical Monthly}, March 2001, 108(3), 232-245.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  If the argument n is not a positive integer that is greater than 1, 
  the function presents an error message and stops.
}
\examples{
F <- frobenius.matrix( 10 )
print( F )
}
\keyword{ math }

\name{vandermonde.matrix}
\alias{vandermonde.matrix}
\title{ Vandermonde matrix }
\description{
  This function returns an m by n matrix of the powers of the alpha vector
}
\usage{
vandermonde.matrix(alpha, n)
}
\arguments{
  \item{alpha}{ A numerical vector of values }
  \item{n}{ The column dimension of the Vandermonde matrix }
}
\details{
  In linear algebra, a Vandermonde matrix is an \eqn{m \times n} matrix with terms
  of a geometric progression of an \eqn{m \times 1} parameter vector \eqn{{\bf{\alpha }} = {\left\lbrack {\begin{array}{cccc}
{{\alpha _1}}&{{\alpha _2}}& \cdots &{{\alpha _m}}
\end{array}} \right\rbrack^\prime }}

  such that \eqn{V\left( {\bf{\alpha }} \right) = \left\lbrack {\begin{array}{ccccc}
1&{{\alpha _1}}&{\alpha _1^2}& \cdots &{\alpha _1^{n - 1}}\\
1&{{\alpha _2}}&{\alpha _2^2}& \cdots &{\alpha _2^{n - 1}}\\
1&{{\alpha _3}}&{\alpha _3^2}& \cdots &{\alpha _3^{n - 1}}\\
 \cdots & \cdots & \cdots & \cdots & \cdots \\
1&{{\alpha _m}}&{\alpha _m^2}& \cdots &{\alpha _m^{n - 1}}
\end{array}} \right\rbrack}.
}
\value{
  A matrix.
}
\references{
  Horn, R. A. and C. R. Johnson (1991). \emph{Topics in matrix analysis}, Cambridge
  University Press.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\examples{
alpha <- c( .1, .2, .3, .4 )
V <- vandermonde.matrix( alpha, 4 )
print( V )
}
\keyword{ math }

\name{fibonacci.matrix}
\alias{fibonacci.matrix}
\title{ Fibonacci Matrix }
\description{
  This function constructs the order n + 1 square Fibonacci matrix
  which is derived from a Fibonacci sequence.
}
\usage{
fibonacci.matrix(n)
}
\arguments{
  \item{n}{ a positive integer value }
}
\details{
  Let \eqn{\left\{ {{f_0},\;{f_1},\; \ldots ,\;{f_n}} \right\}} be the
  set of \eqn{ n + 1} Fibonacci numbers where \eqn{{f_0} = {f_1} = 1}
  and \eqn{{f_j} = {f_{j - 1}} + {f_{j - 2}},\quad 2 \le j \le n}.  The
  order \eqn{n + 1} Fibonacci matrix \eqn{{\bf{F}}} has as typical element
  \eqn{{F_{i,j}} = \left\{ {\begin{array}{cc}
{{f_{i - j + 1}}}&{i - j + 1 \ge 0}\\
0&{i - j + 1 < 0}
\end{array}} \right.}.
}
\value{
  An order \eqn{n + 1} matrix
}
\references{
  Zhang, Z. and J. Wang (2006). Bernoulli matrix and its algebraic properties,
  \emph{Discrete Applied Nathematics}, 154, 1622-1632.
}
\author{ Frederick Novomestky \email{fnovomes@poly.edu} }
\note{
  If the argument n is not a positive integer, the function presents an error message and stops.
}
\examples{
F <- fibonacci.matrix( 10 )
print( F )
}
\keyword{ math }
